/**
 * @class   wosh::devices::MailBoxBundle
 * @brief   Abstraction of a POP3 mailbox, supports SMTP protocol.
 *
 *
 *
 * @ingroup Devices
 ****************************************************************************
 * @version $Id: MailBoxBundle.h 2288 2010-04-09 08:33:26Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Bundles_Devices_MailBox_H__
 #define __WOSH_Bundles_Devices_MailBox_H__

 #include <core/BundleGeneric.h>
 #include <core/Thread.h>
 #include <core/Map.h>

 #include <core/Communicator.h>
 #include <framework/communication/TextMessage.h>
 
 #include <interfaces/Service.h>
 #include <interfaces/services/CommunicatorService.h>
 #include <interfaces/devices/MailBox.h>

 #include "IMailBoxListener.h"


using namespace std;
using namespace wosh::communication;
namespace wosh {
 namespace devices {

#define _MailBox_NAME					"MailBox"
#define _MailBox_VERSION				"0.2"


class MailBoxBundle :	public BundleGeneric,
						public virtual wosh::interfaces::services::CommunicatorService, 
						public virtual wosh::interfaces::devices::MailBox, 
						public virtual IMailBoxListener
 {
	WOSH_BUNDLE(wosh::devices::MailBoxBundle)
	WOSH_BUNDLE_INTERFACE("wosh::devices::MailBox")
	WOSH_BUNDLE_SINGLETON(0,0)


	public:
		/**
		 * @brief   Empty constructor, init bundle and vars, Properties and Methods.
		 */
		MailBoxBundle();
		/**
		 * @brief   Empty destructor, may (synch) stop bundle and internal threads when still running
		 */
		virtual ~MailBoxBundle();


	public:
		WRESULT bundleStart();

		WRESULT bundleStop();

		bool updatingProperty( bool& do_update, const Data& value_proposed, Property& property_current, const PropertiesProvider* source );

		void busMessage( const Message& message, const Bus* source );

		virtual wosh::interfaces::Device::DEVICE_STATE getDeviceState() const {
			return wosh::interfaces::Device::BROKEN;
		 }

//@}
/** @name Communicator Interface
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief   Check if user is online now (logged in current server)
		 * @param username [in] wosh-username to lookup
		 * @return  true if user is online
		 */
		bool isUserReachable( const string& username );

		/**
		 * @brief   Check if user is registered to current server
		 * @param username [in] wosh-username to lookup
		 * @return  true if user is in friends' list
		 */
		bool isUserRegistered( const string& username );

		/**
		 * @brief   Get the timestamp of last(seen) online-status of given user
		 * @param username [in] wosh-username to lookup
		 * @return  epoc timestamp if user is still online, negative epoch if user is actually offline
		 */
		long getUserLastSeen( const string& username );

		/**
		 * @brief   Send a text message to given (wosh) username
		 * @param wosh_username [in] recipent of the message (as WOSH username)
		 * @param message [in] message to be sent
		 * @return WRET_ERR_PARAM if accoutn wasnt' found, WRET_OK when message was sent, WRET_ERR_INTERNAL else.
		 * @see   getAccountNameFromUsername()
		 */
		WRESULT sendMessageToUser( const string& wosh_username, const string& message );

		/**
		 * @brief   Send a text message to given account name [example: asd123@gmail.com]
		 * @param gtalk_username [in] recipent of the message
		 * @param message [in] message to be sent
		 * @return WRET_ERR_PARAM if accoutn wasnt' found, WRET_OK when message was sent, WRET_ERR_INTERNAL else.
		 */
		WRESULT sendMessageTo( const string& gtalk_username, const string& message );

		/**
		 * @brief   Send a Notification translated into a text message
		 * @param notification [in] Notification to communicate
		 * @return WRET_ERR_PARAM if accoutn wasnt' found, WRET_OK when message was sent, WRET_ERR_INTERNAL else.
		 */
		WRESULT notify( const Notification* notification );

//@}
/** @name MailBox Interface - Getters
 * @{
 ******************************************************************************/
	public:
		const std::string& getServerAddress() const;
		int getServerPort() const;
		const std::string& getAccountName() const;
//@}
/** @name MailBox Interface - Setters
 * @{
 ******************************************************************************/
	public:
		bool setServer( const std::string& address, int port );
		bool setAccount( const std::string& username, const std::string& password );
//@}
/** @name IMailBoxListener Events
 * @{
 ******************************************************************************/
 	public:
		void mailbox_Message( MailBoxImpl* mailbox, const string& sender, const string& message );
//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:
		/**
		 * @brief   Retrieve account name from username
		 * @param username [in] wosh-username to lookup
		 * @return account name associated with wosh-username
		 * @see     UserManager::getUserProperty()
		 */
		std::string getEmailFromUsername( const string& username );

		/**
		 * @brief   Retrieve username from email address
		 * @param email_address [in] email address to lookup
		 * @return wosh-username associated with email address
		 * @see     UserManager::findUserByProperty()
		 */
		std::string getUsernameFromEmail( const string& email_address );

		/**
		 * @brief   Create a new Message and inject into Core bus.
		 * @param content [in] the content to be assigned to the message 
		 * @param security [in] the security payload to be assigned to the message 
		 */
		void raiseEvent( MessageContent* content, SecurityToken* security = NULL );

//@}
	private:
		Response* mmDoSendMessageToUser( const Message* requestMessage );
		Response* mmDoSendMail( const Message* requestMessage );
		Response* mmDoListContacts( const Request* request );

	protected:
		BusConnector BusDevices;			///< register myself & work as Device too
		MailBoxImpl* mboxWorker;			///< real implementation (basled on libgloox)
		Map<std::string, long> contacts;	///< negative long value means is actuall offline, use abs() to get epoch

//@}


}; // class def

 }; // namespace devices
}; // namespace wosh

#endif //__WOSH_Bundles_Devices_MailBox_H__
