/**
 * @class   MailBoxImpl
 * @brief   
 *
 ****************************************************************************
 * @version $Id: MailBoxImpl.h 2280 2010-04-07 04:14:56Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Implementations_MailBoxImpl_H__
 #define __WOSH_Implementations_MailBoxImpl_H__

 #include <core/BundleGenericWorker.h>
 #include <core/List.h>

 #include <framework/network/SocketTcp.h>

 #include "IMailBoxListener.h"
 #include "Mail.h"


using namespace std;
using namespace wosh;
/*namespace wosh {
 namespace network {
	class SocketTcp;
 };
};*/
using namespace wosh::network;


class MailBoxImpl :		public wosh::BundleGenericWorker,
						public virtual ISocketTcpListener
 {
	public:
		enum POP3_CONNECTION_STATE {
			NOTCONNECTED,
			AUTHORIZATION,
			TRANSACTION,
			UPDATE
		};

	public:
		/**
		 * @brief   Default constructor. Init vars.
		 * @param bundle [in] the parent bundle, see BundleGenericWorker interface
		 */
		MailBoxImpl( BundleGeneric& bundle );
		/**
		 * @brief   Deconstructor. If thread is stll running, it will be stopped.
		 */
		virtual ~MailBoxImpl();


	public:

		static WRESULT sendMail(	const string& hostname, const string& username, const string& password,
									const string& recipent_address, const string& subject, const string& body );


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief   Set server address and port. Provided for convenience
		 * @param address [in] address of the server
		 * @param port [in] port of the server
		 * @return false when internal thread is running, true else
		 */
		bool setServer( const std::string& address, int port = 110 );

		/**
		 * @brief   Set server address and port. Provided for convenience
		 * @param address [in] address of the server (IP:PORT)
		 * @return false when internal thread is running, true else
		 */
		bool setServerAddress( const std::string& address );

		/**
		 * @brief   Set server port
		 * @param port [in] port of the server
		 * @return false when internal thread is running, true else
		 */
		bool setServerPort( int port );

		/**
		 * @brief   Set server account name and password. Provided for convenience
		 * @param username [in] username used to log on the server
		 * @param password [in] username used to log on the server
		 * @return false when internal thread is running, true else
		 */
		bool setAccount( const std::string& username, const std::string& password );

		/**
		 * @brief   Set account username
		 * @param username [in] username used to log on the server
		 * @return false when internal thread is running, true else
		 */
		bool setAccountUserName( const std::string& username );

		/**
		 * @brief   Set account password
		 * @param password [in] username used to log on the server
		 * @return false when internal thread is running, true else
		 */
		bool setAccountUserPassword( const std::string& password );


		void setAutoCheckFrequency( long seconds )			{ this->autoRefreshState = seconds; }

		void setReadOnly( bool readonly )					{ this->readOnly = readonly; }


		/**
		 * @brief   Set the IMailBoxListener interface for callbacks.
		 * @param listener [in] pointer to object which implements callback-functions
		 * @note    Call this method only once, after creation, it's not reetrant!
		 */
		void setMailBoxListener( IMailBoxListener* listener );

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief   Get server address
		 * @return  address as string
		 */
		inline const std::string& getServerAddress() const	{ return this->pop3ServerAddress; }

		/**
		 * @brief   Get server port
		 * @return  server port
		 */
		inline int getServerPort() const					{ return this->pop3ServerPort; }

		/**
		 * @brief   Get account username
		 * @return  account username
		 */
		inline const std::string& getAccountName() const	{ return this->pop3AccountName; }



		int getMessagesCount()								{ return this->messagesCount; }
		int getMessagesSize()								{ return this->messagesSize; }
		long getStatsTimestamp()							{ return this->statsTimestamp; }

		bool getReadOnly()									{ return this->readOnly; }


		List<Mail*>& getMails()								{ return this->mails; }


		/**
		 * @brief   Get getMailBoxListener interface class for callbacks.
		 * @return  pointer to object which implements callback-functions
		 */
		inline IMailBoxListener* getMailBoxListener()		{ return this->mboxListener; }

//@}
/** @name ISocketTcpListener
 * @{
 ******************************************************************************/
	public:
		virtual void tcpDataAvailable( long data_len, SocketTcp* socket );
		virtual void tcpSocketError( int error_code, const char* description, SocketTcp* socket );

//@}
/** @name Internal methods
 * @{
 ******************************************************************************/
	protected:

		virtual bool initializingThread();
		virtual void runThread();
		virtual void exitingThread();


WRESULT connectServer( const string& host, unsigned int port );

		virtual WRESULT disconnectServer();

		virtual WRESULT login( const string& user, const string& pwd );

		virtual WRESULT doGetStatus(int& nbMessages, int& totalSize);

		WRESULT doGetMsgList( map<int,int>& id_size_list );
		WRESULT doGetMsgUIdList( map<int,string>& id_uid_list );
		WRESULT doGetMsgUId( int msgIndex, string& unique_id );

		virtual WRESULT doDeleteMark( int msgIndex );
		bool doResetMarks();
		bool doNoOperation();

		virtual WRESULT doGetMessageTop( int msgIndex, string& msgTop, unsigned int nbLines = 2 );
		virtual WRESULT doGetMessage( int msgIndex, string& msg );

		WRESULT doGetMessage( int msgIndex, Mail* mail, short how );


		bool isResponseOK( const string& data );

		string doCommand( const string& command, bool isMultiline);

//@}
WRESULT checkMail();
WRESULT readMail();

	protected:

std::string pop3ServerAddress;		///< address of the server
int pop3ServerPort;					///< port of the server

std::string pop3AccountName;		///< Account username of the bot, you should create an account for it
std::string pop3AccountPwd;			///< Account password of the bot, you should create an account for it


		POP3_CONNECTION_STATE state;

		long autoRefreshState;		///< in seconds
		bool readOnly;				///< avoid any changes on server

		bool ready;
		bool handshaked;

		char* msgBufferOut;
		int msgBufferOutSize;
		MutexRW msgBufferOutMux;
		
		char* msgBufferIn;
		int msgBufferInSize;
		//MutexRW msgBufferInMux;

		SocketTcp* tcpSocket;

		List<Mail*> mails;			///<mail list

		int messagesCount;			///< number of messages on server
		int messagesSize;			///< total size of messages (on server)
		long statsTimestamp;		///< epoc timestamp of stats

		IMailBoxListener* mboxListener;	///< callbacks for stats (such as MailBoxBundle*)


}; // class def


#endif //__WOSH_Implementations_MailBoxImpl_H__
