/** @file    PlayerGStreamerBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: PlayerGStreamerBundle.cpp 2814 2010-07-30 06:58:57Z alex $
 *  @brief
 * File containing methods for the wosh::devices::PlayerGStreamerBundle class.
 * The header for this class can be found in PlayerGStreamerBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "PlayerGStreamerBundle.h"
 #include "PlayerGStreamer010.h"

 #include <core/ObjectAllocator.h>
 #include <core/MethodsCommon.h>
 #include <core/MessageFilter.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/Fact.h>
 #include <core/List.h>
 #include <core/Table.h>
 #include <core/DataModels.h>
 #include <core/ThreadManager.h>
 #include <core/DeviceManager.h>
 #include <core/SystemError.h>
 #include <core/WoshKernel.h>


using namespace std;
namespace wosh {
 namespace devices {

 WOSH_REGISTER(wosh::devices::PlayerGStreamerBundle, "wosh::interfaces::devices::PlayerAudio", _PlayerGStreamerBundle_VERSION, _static_PlayerGStreamerBundle1 )
 WOSH_REGISTER(wosh::devices::PlayerGStreamerBundle, "wosh::BundleGeneric", _PlayerGStreamerBundle_VERSION, _static_PlayerGStreamerBundle2 )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

PlayerGStreamerBundle::PlayerGStreamerBundle( const std::string& name, const std::string& device_name )
	: BundleGeneric() {
	if ( name.size() > 0 )	BundleGeneric::setName( name, false );
	else					BundleGeneric::setName( _PlayerGStreamerBundle_NAME, false );
	Properties.update( _KEY_Version, _PlayerGStreamerBundle_VERSION );

Log.setLevel(LOG_VERBOSE);

	this->cPlaying = -1;
	Log(LOG_DEBUG, " Configuring Player worker.." );
	this->playerWorker = new PlayerGStreamer010(*this);
	this->playerWorker->setPlayerListener(this);
	this->playerWorker->setVolume(30);

	this->devState = wosh::interfaces::Device::UNKNOWN;
//	this->playerWorker->setDeviceOutput(device_name);

	Log(LOG_DEBUG, " Setting default properties and permissions.." );
	Properties.set( _Player_KEY_Volume,				30,			Permission::RW_RW_R__() );
	Properties.set( _Player_KEY_DeviceOutput,		device_name,Permission::RW_RW_R__() );
	Properties.set( _Player_KEY_CurrentFile,		"",			Permission( Permission::Read ) );
	Properties.set( _Player_KEY_Mute, 				false,	Permission( Permission::Read ) );
	Properties.set( _Player_KEY_CurrentLength,		"",			Permission( Permission::Read ) );
	Properties.set( _Player_KEY_CurrentPosition,	"",			Permission( Permission::Read ) );
	Properties.set( _Player_KEY_State,				"Idle",		Permission( Permission::Read ) );
	Properties.set( _Player_KEY_RepeatPlayList,		false,	Permission::RW_RW_R__() );
	Properties.set( _Player_KEY_RepeatItem,			false,	Permission::RW_RW_R__() );
	Properties.set( _Device_KEY_DeviceStatus,		wosh::interfaces::Device::getDeviceStateAsString(wosh::interfaces::Device::UNKNOWN), Permission( Permission::Read ) );
	Properties.monitor(_Player_KEY_Volume);
	Properties.monitor(_Player_KEY_DeviceOutput);
	Properties.monitor(_Player_KEY_RepeatItem);
	Properties.monitor(_Player_KEY_RepeatPlayList);

	std::map<std::string, std::string> fields;
	this->playerWorker->getAPIInfo( fields );
	std::map<std::string,std::string>::const_iterator it;
	for ( it=fields.begin(); it!=fields.end(); it++ ) {
		Properties.set( it->first, it->second, Permission( Permission::Read ) );
	 }

	Log(LOG_DEBUG, " Registering methods.." );
	MethodWRESULT* mmPause = Methods.create<MethodWRESULT>( _Player_METHOD_pause, "pause media", Permission::R_XR_XR__() );
	mmPause->setMethod( this, (MethodWRESULTPtr)&PlayerGStreamerBundle::pause ); //direct method
	MethodWRESULT* mmStop = Methods.create<MethodWRESULT>( _Player_METHOD_stop, "stop media", Permission::R_XR_XR__() );
	mmStop->setMethod( this, (MethodWRESULTPtr)&PlayerGStreamerBundle::stop ); //direct method
	MethodWRESULT* mmNext = Methods.create<MethodWRESULT>( _Player_METHOD_next, "play next item", Permission::R_XR_XR__() );
	mmNext->setMethod( this, (MethodWRESULTPtr)&PlayerGStreamerBundle::next ); //direct method
	MethodWRESULT* mmPrev = Methods.create<MethodWRESULT>( _Player_METHOD_prev, "play previous item", Permission::R_XR_XR__() );
	mmPrev->setMethod( this, (MethodWRESULTPtr)&PlayerGStreamerBundle::prev ); //direct method
	MethodWRESULT* mmClear = Methods.create<MethodWRESULT>( _Player_METHOD_clear, "clear playlist", Permission::R_XR_XR__() );
	mmClear->setMethod( this, (MethodWRESULTPtr)&PlayerGStreamerBundle::clear ); //direct method
	MethodWRESULT* mmFirst = Methods.create<MethodWRESULT>( _Player_METHOD_first, "play first item", Permission::R_XR_XR__() );
	mmFirst->setMethod( this, (MethodWRESULTPtr)&PlayerGStreamerBundle::first ); //direct method
	MethodWRESULT* mmLast = Methods.create<MethodWRESULT>( _Player_METHOD_last, "play last item", Permission::R_XR_XR__() );
	mmLast->setMethod( this, (MethodWRESULTPtr)&PlayerGStreamerBundle::last ); //direct method

	MethodList* mmList = Methods.create<MethodList>( _METHOD_List, "List enqueued items", Permission::R_XR_XR__() );
	mmList->setDataFieldExtractor( DataFieldExtractor<MediaTrack>::createFor< DataAdapterListT<MediaTrack> >(this->playerWorker->getPlayList().getTracks(), "Media Tracks") );

	MethodRetrieve* mmRetrieve = Methods.create<MethodRetrieve>( _METHOD_Retrieve, "Retrieve enqueued item(s)", Permission::R_XR_XR__() );
	mmRetrieve->setItemExtractor( ObjectExtractor<MediaTrack, long>::createFor< DataAdapterListT<MediaTrack> >(this->playerWorker->getPlayList().getTracks(), &MediaTrack::getIndex ) );

	MethodRequest* mmPlay = Methods.create<MethodRequest>( _Player_METHOD_play, "play media", Permission::R_XR_XR__() );
	mmPlay->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoPlay );

	MethodRequest* mmPlayIndex = Methods.create<MethodRequest>( _Player_METHOD_play_index, "play item", Permission::R_XR_XR__() );
	mmPlayIndex->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoPlayIndex );

	MethodRequest* mmEnqueue = Methods.create<MethodRequest>( _Player_METHOD_enqueue, "enqueue item", Permission::R_XR_XR__() );
	mmEnqueue->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoEnqueue );

	MethodRequest* mmRemove = Methods.create<MethodRequest>( _Player_METHOD_remove, "remove an item from the list", Permission::R_XR_XR__() );
	mmRemove->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoRemove );

	MethodRequest* mmSetVolume = Methods.create<MethodRequest>( _Player_METHOD_set_volume, "set volume", Permission::RX );
	mmSetVolume->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoSetVolume );

	MethodRequest* mmSetMute = Methods.create<MethodRequest>( _Player_METHOD_set_mute, "(un)set muted", Permission::RX );
	mmSetMute->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoSetMute );

	MethodRequest* mmSetPosition = Methods.create<MethodRequest>( _Player_METHOD_set_position, "set media position", Permission::R_XR_XR__() );
	mmSetPosition->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoSetPosition );

	MethodRequest* mmSynchWithPlayer = Methods.create<MethodRequest>( _Player_METHOD_synch, "synchronize with player", Permission::R_XR_XR__() );
	mmSynchWithPlayer->setMethod( this, (MethodRequestPtr)&PlayerGStreamerBundle::mmDoSynchWithPlayer );

	setBundleState(Bundle::STATE_CREATED, false);
 }

PlayerGStreamerBundle::~PlayerGStreamerBundle() {
	Log(LOG_VERBOSE, " Destroying.." );
	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~PlayerGStreamerBundle() : Destroying while Bundle is running! Trying to stop.." );
		bundleStop();
	 }
	if ( this->playerWorker->isThreadRunning() ) {
		Log(LOG_WARNING, "~PlayerGStreamerBundle() : Destroying while RUNNING! Trying to stop.." );
		this->playerWorker->quitThread(30000);
	 }
	delete this->playerWorker; this->playerWorker = NULL;
	Log(LOG_VERBOSE, ":~PlayerGStreamerBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT PlayerGStreamerBundle::bundleStart() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) ) return WRET_ERR_WRONG_STATE;

	BusDevices.setMessageHandler(this);
	WRESULT bus_connected = BusDevices.connect( _Bus_Devices );
	if ( WFAILED(bus_connected) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "bundleStart", "FAILED#%i Connecting Bus ("_Bus_Devices")..", bus_connected );

	WRESULT ret = BundleGeneric::start_SynchThread( this->playerWorker );
	if ( WSUCCEEDED(ret) ) {
		Log(LOG_VERBOSE, ":bundleStart() : Registering as Device.." );
		wosh::Device* myself = new wosh::Device(*this);
		myself->getInterfaces() = this->Interfaces;
		WoshKernel::getInstance()->devices().registerDevice( myself );
	 }
	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: IThreadListener::thread_event()
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT PlayerGStreamerBundle::bundleStop() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) ) return WRET_ERR_WRONG_STATE;

	WRESULT ret = WRET_OK;
	if ( this->playerWorker->isPlaying() )
		ret += this->playerWorker->stop();
	ret += this->playerWorker->clear();

	ret += BundleGeneric::stop_SynchThread( this->playerWorker );
	if ( WSUCCEEDED(ret) ) {
		Log(LOG_VERBOSE, ":bundleStop() : UnRegistering as Device.." );
		WoshKernel::getInstance()->devices().unRegisterDevice( this->getName() );
	 }
	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: IThreadListener::thread_event()
	return ret;
 }

WRESULT PlayerGStreamerBundle::bundleDiagnostic() {
	Log(LOG_VERBOSE, ":bundleDiagnostic() : Starting PlayerGStreamerBundle-Diagnostic" );

	WRESULT ret = BundleGeneric::bundleDiagnostic();

	Log(LOG_INFO, ":bundleDiagnostic() : Finished PlayerGStreamerBundle-Diagnostic [%i]", ret );
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// BUS MESSAGES

void PlayerGStreamerBundle::busMessage( const Message& message, const Bus* source ) {
	DeviceManager::busMessageForBundleDevices( message, source, this, this->Methods, this->Log, this->BusCore, this->BusDevices );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

void PlayerGStreamerBundle::raiseEvent( MessageContent* fact ) {
	if ( fact == NULL ) return;
	Message* msg_event = new Message(fact);
	msg_event->setDestinationBroadcast();
	signMessage(msg_event);
	BusDevices.postMessage(msg_event);
 }

//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS

bool PlayerGStreamerBundle::readingProperty( Property* property_curr, const PropertiesProvider* ) {
	if ( property_curr == NULL ) return true;
	if ( property_curr->getKey() == _Player_KEY_CurrentPosition ) {
		if ( this->playerWorker->isPlaying() )
			property_curr->getValue().set( this->playerWorker->getPositionMs(), Variant::TIMESPAN );
		else
			property_curr->getValue().set( 0L, Variant::TIMESPAN );
	 }
	return true;
 }

bool PlayerGStreamerBundle::updatingProperty( bool& do_update, const Variant& value_proposed, Property& property_current, const PropertiesProvider* source ) {
	if ( property_current.getKey() == _Player_KEY_Volume ) {
		WRESULT ret = setVolume( value_proposed.toInteger(-1) );
		return WSUCCEEDED(ret);
	 }
	else if ( property_current.getKey() == _Player_KEY_DeviceOutput ) {
		WRESULT ret = setDeviceOutput(value_proposed.toString());
		return WSUCCEEDED(ret);
	 }
	else if ( property_current.getKey() == _Player_KEY_RepeatItem ) {
		setRepeatItem(value_proposed.toBoolean(false));
		return WRET_OK;
	 }
	else if ( property_current.getKey() == _Player_KEY_RepeatPlayList ) {
		setRepeatPlayList(value_proposed.toBoolean(false));
		return WRET_OK;
	 }
	else // forward processing to base class
		return BundleGeneric::updatingProperty(do_update, value_proposed, property_current, source);
 }

///////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// THREAD EVENTS

void PlayerGStreamerBundle::thread_event( Thread::THREAD_STATE thread_event, Thread* thread_source ) {
	if ( thread_source != NULL )
		Log(LOG_DEBUG, ":thread_event(%i, %ld) : [%s] Raised state %s", (int)thread_event, thread_source->getThreadID(), thread_source->getThreadName().c_str(), Thread::getThreadStateAsString(thread_event) );
	else
		Log(LOG_DEBUG, ":thread_event(%i,NULL) UNKNOWN thread raised state %s", (int)thread_event, Thread::getThreadStateAsString(thread_event) );
	switch(thread_event) {
		case Thread::STATE_RUNNING: {
				setBundleState( Bundle::STATE_STARTED );
				Properties.update( _Device_KEY_DeviceStatus, wosh::interfaces::Device::getDeviceStateAsString(wosh::interfaces::Device::LINKED) );
				break;
			 }
		case Thread::STATE_STOPPED: {
				setBundleState( Bundle::STATE_STOPPED );
				Properties.update( _Device_KEY_DeviceStatus, wosh::interfaces::Device::getDeviceStateAsString(wosh::interfaces::Device::UNLINKED) );
				break;
			 }
		case Thread::STATE_CRASHED:
		case Thread::STATE_FAILURE: {
				setBundleState( Bundle::STATE_STOPPED );
				Properties.update( _Device_KEY_DeviceStatus, wosh::interfaces::Device::getDeviceStateAsString(wosh::interfaces::Device::BROKEN) );
				break;
			 }
		default: break;
	 }
 }

/////////////////////////////////////////////////////////////////////////////////////////////// THREAD EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// PLAYER EVENTS

void PlayerGStreamerBundle::changed_PlayState( int playing, PlayerGStreamer* ) {
	string status = PlayerAudioEvent_StateChanged::getPlayStateAsString(playing);
	Properties.transactionBeginWrite();
	Properties.update( _Player_KEY_State, status );
	Properties.transactionEnd();
	if ( this->cPlaying != playing ) // status changed
		raiseEvent( new PlayerAudioEvent_StateChanged(playing, this->playerWorker->getVolume(), this->playerWorker->isMuted()) );
	this->cPlaying = playing;
 }

void PlayerGStreamerBundle::changed_Volume( int volume, bool muted, PlayerGStreamer* ) {
	Properties.transactionBeginWrite();
	Properties.update( _Player_KEY_Mute, muted );
	Properties.update( _Player_KEY_Volume, volume );
	Properties.transactionEnd();
	raiseEvent( new PlayerAudioEvent_StateChanged(this->playerWorker->isPlaying(), volume, muted) );
 }

void PlayerGStreamerBundle::changed_PlayList( const MediaList* list, PlayerGStreamer* ) {
//	raiseEvent( new PlayerAudioEvent_MediaListChanged(list) );
 }

void PlayerGStreamerBundle::changed_PlayItem( const MediaTrack* track, PlayerGStreamer* ) {
	Properties.transactionBeginWrite();
	if ( track == NULL ) {
		Properties.update( _Player_KEY_CurrentFile, "" );
		Properties.update( _Player_KEY_CurrentLength, "" );
		Properties.update( _Player_KEY_CurrentPosition, "" );
	 }
	else {
		Properties.update( _Player_KEY_CurrentFile, track->getTrack().getPathFull() );
		Properties.update( _Player_KEY_CurrentLength, track->getProperties().getValueOf(_MediaTrack_KEY_Length) );
	 }
	Properties.transactionEnd();
//	raiseEvent( new PlayerAudioEvent_MediaTrackChanged(track) );
 }

/////////////////////////////////////////////////////////////////////////////////////////////// PLAYER EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PLAYER CONTROL

WRESULT PlayerGStreamerBundle::play()			{ return this->playerWorker->play(); }
WRESULT PlayerGStreamerBundle::pause()			{ return this->playerWorker->pause(); }
WRESULT PlayerGStreamerBundle::stop()			{ return this->playerWorker->stop(); }
WRESULT PlayerGStreamerBundle::clear()			{ return this->playerWorker->clear(); }
WRESULT PlayerGStreamerBundle::first()			{ return this->playerWorker->first(); }
WRESULT PlayerGStreamerBundle::next()			{ return this->playerWorker->next(); }
WRESULT PlayerGStreamerBundle::prev()			{ return this->playerWorker->prev(); }
WRESULT PlayerGStreamerBundle::last()			{ return this->playerWorker->last(); }

int PlayerGStreamerBundle::getVolume()			{ return this->playerWorker->getVolume(); }
bool PlayerGStreamerBundle::isMuted()			{ return this->playerWorker->isMuted(); }
bool PlayerGStreamerBundle::isPlaying()			{ return this->playerWorker->isPlaying(); }
int PlayerGStreamerBundle::getPlaylistCount()	{ return this->playerWorker->getPlayList().size(); }

WRESULT PlayerGStreamerBundle::play( unsigned int index )
												{ return this->playerWorker->play(index); }
WRESULT PlayerGStreamerBundle::play( const std::string& file_url )
												{ return this->playerWorker->play(file_url); }
WRESULT PlayerGStreamerBundle::enqueue( const std::string& file_url )
												{ return this->playerWorker->enqueue(file_url); }

WRESULT PlayerGStreamerBundle::remove( unsigned int index )
												{ return this->playerWorker->remove(index); }

void PlayerGStreamerBundle::setRepeatItem( bool value )
												{ return this->playerWorker->setRepeatItem(value); }
void PlayerGStreamerBundle::setRepeatPlayList( bool value )
												{ return this->playerWorker->setRepeatPlayList(value); }

WRESULT PlayerGStreamerBundle::setDeviceOutput( const std::string& dev_file )
												{ return this->playerWorker->setDeviceOutput(dev_file); }
WRESULT PlayerGStreamerBundle::setVolume( int newVolume )
												{ return this->playerWorker->setVolume(newVolume); }
WRESULT PlayerGStreamerBundle::setVolumeOffset( int volume_span )
												{ return this->playerWorker->setVolumeOffset(volume_span); }

WRESULT PlayerGStreamerBundle::setMute( bool on )
												{ return this->playerWorker->setMute(on); }
WRESULT PlayerGStreamerBundle::setPosition( float position_rate )
												{ return this->playerWorker->setPosition(position_rate); }
WRESULT PlayerGStreamerBundle::setPositionOffset( float pos_off )
												{ return this->playerWorker->setPositionOffset(pos_off); }

////////////////////////////////////////////////////////////////////////////////////////////// PLAYER CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////// OTHER METHODS

WRESULT PlayerGStreamerBundle::synchWithPlayer( const std::string& device_player_name, int flags ) {
	(void)flags;
	MediaList* ml_copy = this->playerWorker->getPlayList().clone();
	ml_copy->setCurrentPosition( this->playerWorker->getPositionMs() );

	Message* message = new Message();
	Request* msg_req = new Request( _Player_METHOD_synch, new Variant(ml_copy) );
	message->setSource(this);
	message->setDestinationBroadcast();
	message->getDestination().setName(device_player_name);
	message->setContent( msg_req );

	BusDevices.postMessage( message );
	return WRET_OK;
 }

WRESULT PlayerGStreamerBundle::synchFromPlayList( const MediaList* medialist ) {
	this->playerWorker->getPlayList() = *medialist;
	this->playerWorker->play( medialist->getCurrentIndex() );
	this->playerWorker->setPositionMs( medialist->getCurrentPosition() );
	return WRET_OK;
 }

/////////////////////////////////////////////////////////////////////////////////////////////// OTHER METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* PlayerGStreamerBundle::mmDoEnqueue( const Request* request ) {
	if ( request == NULL ) return NULL;
	string fileStr = request->getArgument().toString();
	Log(LOG_DEBUG, ":mmDoEnqueue(%s)", fileStr.c_str() );
	WRESULT ret = enqueue(fileStr);
	return request->replyResponse(ret, new Variant(this->playerWorker->getPlayList().size()) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* PlayerGStreamerBundle::mmDoRemove( const Request* request ) {
	if ( request == NULL ) return NULL;
	string option = request->getArgument().toString();
	int index = Utilities::fromString<int>(option);
	Log(LOG_DEBUG, ":mmDoRemove(%i)", index );
	WRESULT ret = remove( (unsigned int)index );
	return request->replyResponse(ret, new Variant(index) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* PlayerGStreamerBundle::mmDoPlayIndex( const Request* request ) {
	if ( request == NULL ) return NULL;
	int index = request->getArgument().toInteger(-1);
	Log(LOG_DEBUG, ":mmDoPlayIndex(%i)", index );
	WRESULT ret = play(index);
	return request->replyResponse(ret, new Variant(index) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* PlayerGStreamerBundle::mmDoPlay( const Request* request ) {
	if ( request == NULL ) return NULL;
	string fileStr = request->getArgument().toString();
	Log(LOG_DEBUG, ":mmDoPlay(%s)", fileStr.c_str() );
	WRESULT ret = play(fileStr);
	return request->replyResponse(ret);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* PlayerGStreamerBundle::mmDoSynchWithPlayer( const Request* request ) {
	if ( request == NULL ) return NULL;
	WRESULT ret = WRET_OK;
	if ( !request->hasData() ) return NULL;
	if ( request->getData()->isKindOf<MediaList>() ) {
		const MediaList* medialist = request->getData()->as<MediaList>();
		if ( medialist == NULL )
			return request->replyResponse(WRET_ERR_PARAM);
		Log(LOG_INFO, ":mmDoSynchWithPlayer(%s) %i items, play %i @ %i", "TODO", medialist->size(),
																				medialist->getCurrentIndex(),
																				medialist->getCurrentPosition()  );
		ret = synchFromPlayList(medialist);
	 }
	else if ( request->getData()->isKindOf<Variant>() ) {
		string device = request->getArgument().toString();
		if (device.size() == 0 )
			return request->replyResponse(WRET_ERR_PARAM );
		Log(LOG_INFO, ":mmDoSynchWithPlayer(%s)", device.c_str() );
		ret = synchWithPlayer(device);
	 }
	return request->replyResponse(ret);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* PlayerGStreamerBundle::mmDoSetVolume( const Request* request ) {
	if ( request == NULL ) return NULL;
	string volumeStr = request->getArgument().toString();
	Utilities::trimSpaces(volumeStr);
	Log(LOG_DEBUG, ":mmDoSetVolume(%s)", volumeStr.c_str() );
	if ( volumeStr.size() == 0 )
		return request->replyResponse(WRET_ERR_PARAM);
	WRESULT ret = WRET_OK;
	if (volumeStr.substr(0,1) == "+" || volumeStr.substr(0,1) == "-") {
		int sign = 1;
		if ( volumeStr.substr(0,1) == "-" )
			sign = -1;

		int vol_span = 2;
		if ( volumeStr.size() > 1 ) {
			vol_span = Utilities::fromString<int>(volumeStr.substr(1));
		 }
		if ( vol_span <= 0 )
			vol_span = 2;
		ret = setVolumeOffset( vol_span * sign );
	 }
	else
		ret = setVolume( Utilities::fromString<int>(volumeStr) );
	return request->replyResponse(ret, new Variant(getVolume()) );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* PlayerGStreamerBundle::mmDoSetMute( const Request* request ) {
	if ( request == NULL ) return NULL;
	if ( request->getArgument().isEmpty() )
		return request->replyResponse(WRET_ERR_PARAM, "Invalid Param [bool]");
	bool mute = request->getArgument().toBoolean(false);
	Log(LOG_DEBUG, ":mmDoSetMute(%i)", mute );
	WRESULT ret = setMute( mute );
	return request->replyResponse(ret);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* PlayerGStreamerBundle::mmDoSetPosition( const Request* request ) {
	if ( request == NULL ) return NULL;
	string posStr = request->getArgument().toString();
	Utilities::trimSpaces(posStr);
	Log(LOG_DEBUG, ":mmDoSetPosition(%s)", posStr.c_str() );
	WRESULT ret = WRET_OK;
	float pos = -1.0;
	if ( posStr == "+" )
		ret = setPositionOffset(0.1F);
	else if ( posStr == "-" )
		ret = setPositionOffset(-0.1F);
	else {
		pos = Utilities::fromString<float>(posStr, -1.0);
		ret = setPosition(pos);
	 }
	return request->replyResponse(ret);
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace devices
}; // namespace wosh
