/** @file    GardenerBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: GardenerBundle.cpp 2732 2010-07-06 03:18:11Z alex $
 *  @brief
 * File containing methods for the wosh::services::GardenerBundle class.
 * The header for this class can be found in GardenerBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "GardenerBundle.h"

 #include <core/ObjectAllocator.h>
 #include <core/Fact.h>
 #include <core/Table.h>

 #include <core/MessageProcessingCommon.h>
 #include <core/SystemError.h>
 #include <core/WoshKernel.h>


using namespace std;
namespace wosh {
 namespace services {

 WOSH_REGISTER(wosh::services::GardenerBundle, "wosh::interfaces::services::Gardener", _Gardener_VERSION, _static_GardenerBundle1 )
 WOSH_REGISTER(wosh::services::GardenerBundle, "wosh::BundleGeneric", _Gardener_VERSION, _static_GardenerBundle2 )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

GardenerBundle::GardenerBundle()
	: BundleGeneric() {
	BundleGeneric::setName( _Gardener_NAME, false );
	Properties.update( _KEY_Version, _Gardener_VERSION );

//Log.setLevel(LOG_VERBOSE);

	this->objPermission.setOwner(_Gardener_USER);
	this->objPermission.setGroup("root");

	Log(LOG_DEBUG, " Configuring worker.." );
	this->gardenerWorker = new GardenerWorker(*this);
	this->gardenerWorker->setListener(this);

	Log(LOG_DEBUG, " Setting default properties and permissions.." );

	Log(LOG_DEBUG, " Registering methods.." );

	setBundleState(Bundle::STATE_CREATED, false);
 }


GardenerBundle::~GardenerBundle() {
	Log(LOG_INFO, " Destroying.." );
	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~GardenerBundle() : Destroying while Bundle is running! Trying to stop.." );
		bundleStop();
	 }
	delete this->gardenerWorker; this->gardenerWorker = NULL;
	Log(LOG_VERBOSE, ":~GardenerBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT GardenerBundle::bundleStart() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) ) return WRET_ERR_WRONG_STATE;

	BusBuilding.setMessageHandler(this);
	WRESULT bus_b_connected = BusBuilding.connect( _Bus_Building, true ); // create bus if missing
	if ( WFAILED(bus_b_connected) )
		SystemError::raise(this, Log, SystemError::CRITICAL, 0, "bundleStart", "FAILED#%i Connecting Bus (wosh.Bus.Building)..", bus_b_connected );

	WRESULT ret = BundleGeneric::start_SynchThread( this->gardenerWorker );
	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT GardenerBundle::bundleStop() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) ) return WRET_ERR_WRONG_STATE;

	WRESULT bus_b_disconnected = BusBuilding.disconnect();
	if ( WFAILED(bus_b_disconnected) )
		SystemError::raise(this, Log, SystemError::WARNING, 0, "bundleStart", "FAILED#%i Disconnecting Bus (wosh.Bus.Building)..", bus_b_disconnected );

	WRESULT ret = BundleGeneric::stop_SynchThread( this->gardenerWorker );
	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()

	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void GardenerBundle::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;
	BundleGeneric::busMessage(message, source);
	
	if ( message.getContent()->isFact() ) {
		const wosh::Fact* fact = message.getContent()->asFact();
		if ( fact->getEvent() == _WeatherInfo_EVENT_WeatherConditionUpdated ) {
			const wosh::xtended::WeatherCondition_Updated* wc_news = message.getContent()->as<wosh::xtended::WeatherCondition_Updated>();
			this->gardenerWorker->evaluateWeatherUpdate(wc_news->getWeatherInfo(), wc_news->getSimilarity() );
		 }
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT


////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh
