/** @file    HumanInterpreterBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: HumanInterpreterBundle.cpp 2809 2010-07-30 05:20:06Z alex $
 *  @brief
 * File containing methods for the wosh::HumanInterpreterBundle class.
 * The header for this class can be found in HumanInterpreterBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "HumanInterpreterBundle.h"

 #include "FreeLingImpl.h"

 #include <core/MessageProcessingCommon.h>


using namespace std;
namespace wosh {
 namespace services {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

HumanInterpreterBundle::HumanInterpreterBundle()
	: BundleGeneric() {
	BundleGeneric::setName( _HumanInterpreter_NAME );
	Properties.update( _KEY_Version, _HumanInterpreter_VERSION );

//Log.setLevel(LOG_VERBOSE);

	Log(LOG_DEBUG, " Configuring Festival worker.." );
	this->autoWorker = new FreeLingImpl(*this);

	Log(LOG_DEBUG, " Setting default properties and permissions.." );
	Properties.set( _InterpreterService_KEY_InputTypes,		(int)Notification::TYPE_TEXTUAL, Permission(Permission::Read) );
	Properties.set( _InterpreterService_KEY_OutputTypes,	(int)Notification::TYPE_TEXTUAL, Permission(Permission::Read) );
	Properties.set( _InterpreterService_KEY_InputLanguages,	"en_GB", Permission(Permission::Read) );
	Properties.set( _InterpreterService_KEY_OutputLanguages,"en_GB", Permission(Permission::Read) );
	Properties.set( _InterpreterService_KEY_InputProtocols,	(int)Notification::PROTOCOL_SHELL + (int)Notification::PROTOCOL_HUMAN, Permission(Permission::Read) );
	Properties.set( _InterpreterService_KEY_OutputProtocols,(int)Notification::PROTOCOL_SHELL + (int)Notification::PROTOCOL_HUMAN, Permission(Permission::Read) );
	
	Log(LOG_DEBUG, " Registering methods.." );
	MethodMessageResponse* mmConvert = Methods.create<MethodMessageResponse>( _InterpreterService_METHOD_convert, "convert message" );
	mmConvert->setMethod( this, (MethodMessageResponsePtr)&HumanInterpreterBundle::mmDoConvert );
	mmConvert->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	setBundleState(Bundle::STATE_CREATED, false);
 }


HumanInterpreterBundle::~HumanInterpreterBundle() {
	Log(LOG_INFO, " Destroying.." );
	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~HumanInterpreterBundle() : Destroying while Bundle is running! Trying to stop.." );
		bundleStop();
	 }
	delete this->autoWorker; this->autoWorker = NULL;
	Log(LOG_VERBOSE, ":~HumanInterpreterBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT HumanInterpreterBundle::bundleStart() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) ) return WRET_ERR_WRONG_STATE;
	WRESULT ret = WRET_OK;

	setBundleState( Bundle::STATE_STARTED );
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT HumanInterpreterBundle::bundleStop() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) ) return WRET_ERR_WRONG_STATE;
	WRESULT ret = WRET_OK;

	setBundleState( Bundle::STATE_STOPPED );
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool HumanInterpreterBundle::validateTranslation( const NotificationConversion* conversion ) {
	if ( conversion == NULL ) return false;
	if ( !conversion->hasOriginal() ) return false;
	const Notification* notif = conversion->getOriginal();

	// verify i'm able to convert
	if ( conversion->getNotificationType() != Notification::TYPE_UNKNOWN && conversion->getNotificationType() != Notification::TYPE_TEXTUAL ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Target type [%i]", notif->getID(), (int)conversion->getNotificationType() );
		return false;
	 }
	if ( notif->getNotificationType() != Notification::TYPE_TEXTUAL ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Input type [%i=%i]", notif->getID(), -1, (int)conversion->getNotificationType() );
		return false;
	 }

	// i'm not able to translate, and i understand english language
	if ( notif->getLanguage() != "en_GB" ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Input Language [%s!=en_GB => %s]", notif->getID(), notif->getLanguage().c_str(), conversion->getLanguage().c_str() );
		return false;
	 }
	if ( !conversion->getLanguage().empty() && conversion->getLanguage() != "en_GB" ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Output Language [%s => %s!=en_GB]", notif->getID(), notif->getLanguage().c_str(), conversion->getLanguage().c_str() );
		return false;
	 }

	if ( conversion->getOriginal()->getPlain().empty() ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Text is EMPTY", notif->getID() );
		return false;
	 }

	return true;
 }

WRESULT HumanInterpreterBundle::enqueueTranslation( const NotificationConversion* conversion, const URI* uri ) {
	if ( conversion == NULL ) return WRET_ERR_PARAM;
	FreeLingJob* job = new FreeLingJob();
	job->conversion = conversion->clone();
	job->requestSource = *uri;
	this->autoWorker->enqueue(job);
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS

void HumanInterpreterBundle::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;

	if ( message.getContent()->isNotification() ) {
		if ( message.getDestination().isBroadcast() )
			return;
		if ( !MessageFilter::isMessageTarget(message, this) )
			return;
		const Notification* notif = message.getContent()->asNotification();

		return;
	 }
	BundleGeneric::busMessage(message, source);
 }

//////////////////////////////////////////////////////////////////////////////////////////// GENERATED EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
WRESULT HumanInterpreterBundle::translateHumanToShell( const string& input_english, string& output_shell )
 {
	if ( input_english == "list" )
		output_shell = "list";

	else
		output_shell = input_english + "\nHUMAN2SHELL!";
	return WRET_OK;
 }

WRESULT HumanInterpreterBundle::translateShellToHuman( const string& input_shell, string& output_english )
 {
	output_english = input_shell + "\nShellToHuman!";
	return WRET_OK;
 }
*/

Notification* HumanInterpreterBundle::translate( const NotificationConversion* conversion )
 {
	if ( conversion == NULL ) return NULL;
	if ( !conversion->hasOriginal() ) return NULL;
	const Notification* notif = conversion->getOriginal();

	// verify i'm able to convert
	if ( conversion->getNotificationType() != Notification::TYPE_UNKNOWN && conversion->getNotificationType() != Notification::TYPE_TEXTUAL ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Target type [%i]", notif->getID(), (int)conversion->getNotificationType() );
		return NULL;
	 }
	if ( notif->getNotificationType() != Notification::TYPE_TEXTUAL ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Input type [%i=%i]", notif->getID(), -1, (int)conversion->getNotificationType() );
		return NULL;
	 }

	// i'm not able to translate, and i understand english language
	if ( notif->getLanguage() != "en_GB" ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Input Language [%s!=en_GB => %s]", notif->getID(), notif->getLanguage().c_str(), conversion->getLanguage().c_str() );
		return NULL;
	 }
	if ( !conversion->getLanguage().empty() && conversion->getLanguage() != "en_GB" ) {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Output Language [%s => %s!=en_GB]", notif->getID(), notif->getLanguage().c_str(), conversion->getLanguage().c_str() );
		return NULL;
	 }

	Notification* outNotif = NULL;
	// CASE A
	//  input is a plain text in english language => PARSE (and generate shell command)
	//  output is a SHELL-syntax notification (message)
	if ( notif->getProtocol() == Notification::PROTOCOL_HUMAN && conversion->getProtocol() == Notification::PROTOCOL_SHELL ) {
		Log(LOG_VERBOSE, ":translate(%ld) : Processing HumanToShell", notif->getID() );
		outNotif = notif->clone();
		string output = "";
//		translateHumanToShell( notif->getPlain(), output );
		outNotif->setPlain(output);
		outNotif->setProtocol(Notification::PROTOCOL_SHELL);
	 }
	// CASE B
	//  input is a SHELL-syntax notification (message) => PARSE (and generate human language)
	//  output is a plain text in english language
	else if ( notif->getProtocol() == Notification::PROTOCOL_SHELL && conversion->getProtocol() == Notification::PROTOCOL_HUMAN ) {
		Log(LOG_VERBOSE, ":translate(%ld) : Processing ShellToHuman", notif->getID() );
		outNotif = notif->clone();
		string output = "";
//		translateShellToHuman( notif->getPlain(), output );
		outNotif->setPlain(output);
		outNotif->setProtocol(Notification::PROTOCOL_HUMAN);
	 }
	else {
		Log(LOG_CRITICAL, ":translate(%ld) : Invalid Protocol [%i]", notif->getID(), (int)notif->getProtocol() );
	 }

	// 

	return outNotif;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS

Response* HumanInterpreterBundle::mmDoConvert( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	if ( !request->hasData() || !request->getData()->isKindOf<NotificationConversion>() ) {
		return new Response( request->getMethod(), WRET_ERR_PARAM, "Wrong DataType: not NotificationConversion!" );
	 }
	const NotificationConversion* conv = dynamic_cast<const NotificationConversion*>( request->getData() );
	if ( !conv->hasOriginal() ) {
		return new Response( request->getMethod(), WRET_ERR_PARAM, "Original Notification missing!" );
	 }
	Log(LOG_VERBOSE, ":mmDoConvert(%ld) TYPE:%i PROTO:%i [%s]", conv->getOriginal()->getID(),
									(int)conv->getNotificationType(), (int)conv->getProtocol(), conv->getLanguage().c_str() );

	bool valid = validateTranslation( conv );
	if ( !valid )
		return new Response( request->getMethod(), WRET_ERR_PARAM, "Invalid source/conversion" );

	WRESULT ret = enqueueTranslation( conv, &requestMessage->getSource() );

	return new Response( request->getMethod(), ret, "enqueued" );
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// METHODS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh

