/**
 * @file    test_FreeLing.cpp
 * @brief   Test case for FreeLing
 *
 * @verbatim

 g++ test_FreeLing.cpp -I/usr/include -L/usr/lib -lpcre -lomlet -lfries -lmorfo -ldb-4.6 -ldb_cxx-4.6 -lboost_filesystem

@endverbatim
 *
 ****************************************************************************
 * @version $Id: test_FreeLing.cpp 1708 2010-01-26 22:28:01Z alex $
 * @author Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

 #include <string>

 #include <freeling.h>

 void ProcessResults(const list<sentence> &ls);


int main()
 {

	//------------------------------------------------------------------------------
	// INIT

	string path="/usr/share/FreeLing/en/";
	// create analyzers
	tokenizer tk(path+"tokenizer.dat"); 
	splitter sp(path+"splitter.dat");

	// morphological analysis has a lot of options, and for simplicity they are packed up
	// in a maco_options object. First, create the maco_options object with default values.
	maco_options opt("es");  
	// then, set required options on/off  
	opt.QuantitiesDetection = false;  //deactivate ratio/currency/magnitudes detection 
	opt.AffixAnalysis = true; opt.MultiwordsDetection = true; opt.NumbersDetection = true; 
	opt.PunctuationDetection = true; opt.DatesDetection = true; opt.QuantitiesDetection = false; 
	opt.DictionarySearch = true; opt.ProbabilityAssignment = true; opt.NERecognition = NER_BASIC;   
	// alternatively, you can set active modules in a single call:
	//     opt.set_active_modules(true, true, true, true, true, false, true, true, NER_BASIC);

	// and provide files for morphological submodules. Note that it is not necessary
	// to set opt.QuantitiesFile, since Quantities module was deactivated.
	opt.LocutionsFile=path+"locucions.dat"; opt.AffixFile=path+"afixos.dat";
	opt.ProbabilityFile=path+"probabilitats.dat"; opt.DictionaryFile=path+"maco.db";
	opt.NPdataFile=path+"np.dat"; opt.PunctuationFile=path+"../common/punct.dat"; 
	// alternatively, you can set the files in a single call:
	//  opt.set_data_files(path+"locucions.dat", "", path+"afixos.dat", 
	//                     path+"probabilitats.dat", path+"maco.db", 
	//                     path+"np.dat", path+"../common/punct.dat");

	// create the analyzer with the just build set of maco_options
	maco morfo(opt); 
	// create a hmm tagger for spanish (with retokenization ability, and forced 
	// to choose only one tag per word)
	hmm_tagger tagger("es", path+"tagger.dat", true, true); 
	// create chunker
	chart_parser parser(path+"grammar-dep.dat");
	// create dependency parser 
	dep_txala dep(path+"dep/dependences.dat", parser.get_start_symbol());

	// INIT
	//------------------------------------------------------------------------------

	string text;
	list<word> lw;
	list<sentence> ls;
	printf("type text: [QUIT to exit]:");
	// get plain text input lines while not EOF.
	while (getline(cin,text)) {
		if ( text == "QUIT" )
			break;

		// tokenize input line into a list of words
		lw=tk.tokenize(text);

		// accumulate list of words in splitter buffer, returning a list of sentences.
		// The resulting list of sentences may be empty if the splitter has still not 
		// enough evidence to decide that a complete sentence has been found. The list
		// may contain more than one sentence (since a single input line may consist 
		// of several complete sentences).
		ls=sp.split(lw, false);

		// perform and output morphosyntactic analysis and disambiguation
		morfo.analyze(ls);
		tagger.analyze(ls);

		// Do whatever our application does with the analyzed sentences
		ProcessResults(ls);

		// clear temporary lists;
		lw.clear(); ls.clear();    
	}

	// No more lines to read. Make sure the splitter doesn't retain anything  
	ls=sp.split(lw, true);   

	// analyze sentence(s) which might be lingering in the buffer, if any.
	morfo.analyze(ls);
	tagger.analyze(ls);

	// Process last sentence(s)
	ProcessResults(ls);


	return 0;
 }



void ProcessResults(const list<sentence> &ls) {
  
  list<sentence>::const_iterator s;
  word::const_iterator a;   //iterator over all analysis of a word
  sentence::const_iterator w;
  
  // for each sentence in list
  for (s=ls.begin(); s!=ls.end(); s++) {
    
    // print sentence XML tag
    cout<<"<SENT>"<<endl;
      
    // for each word in sentence
    for (w=s->begin(); w!=s->end(); w++) {
      
      // print word form, with PoS and lemma chosen by the tagger
      cout<<"  <WORD form=\""<<w->get_form();
      cout<<"\" lemma=\""<<w->get_lemma();
      cout<<"\" pos=\""<<w->get_parole();
      cout<<"\">"<<endl;
      
      // for each possible analysis in word, output lemma, parole and probability
      for (a=w->analysis_begin(); a!=w->analysis_end(); ++a) {
	
        // print analysis info
        cout<<"    <ANALYSIS lemma=\""<<a->get_lemma();
        cout<<"\" pos=\""<<a->get_parole();
        cout<<"\" prob=\""<<a->get_prob();
        cout<<"\"/>"<<endl;
      }
      
      // close word XML tag after list of analysis
      cout<<"</WORD>"<<endl;
    }
    
    // close sentence XML tag
    cout<<"</SENT>"<<endl;
  }
}
























