/**
 * @file    test_Wordnet.cpp
 * @brief   Test case for Wordnet
 *
 * @verbatim

 g++ test_Wordnet.cpp -I/usr/include -L/usr/lib -lwordnet
 ./a.out room kitchen

@endverbatim
 *
 *
 * http://trec.nist.gov/pubs/trec11/papers/pohang.seunghoon.pdf
 *
 ****************************************************************************
 * @version $Id: test_Wordnet.cpp 1708 2010-01-26 22:28:01Z alex $
 * @author Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#include <iostream>
#include <sstream>
#include <string>
#include <cstring>
#include <vector>
#include <algorithm>
#include <wn.h>

#define 	WN_THRESHOLD 	10


using namespace std;

typedef enum  {
	antonym = 1,
	hypernym,
	hyponym,
	entailnym,
	simnym,
	meronym = 12,
	holonym
} wn_rel;

class CommonPath  {
	long int common_index1;
	long int common_index2;

public:
	vector<Synset> path;
	long int distance;

	Synset from() const  { return path[0]; }
	Synset to() const  { return path[path.size()-1]; }

	CommonPath (vector<Synset> syn1, vector<Synset> syn2)  {
		common_index1 = -1;
		common_index2 = -1;
		distance = -1;

		for (size_t i=0; i < syn1.size() && common_index1 == -1 && i < WN_THRESHOLD; i++)  {
			for (size_t j=0; j < syn2.size() && common_index1 == -1 && j < WN_THRESHOLD; j++)  {
				if (syn1[i].hereiam == syn2[j].hereiam)  {
					common_index1 = i;
					common_index2 = j;
				}
			}
		}

		if (common_index1 >= 0 && common_index2 >= 0)  {
			for (long int i=0; i < common_index1; i++)
				path.push_back(syn1[i]);

			for (long int i = common_index2; i >= 0; i--)
				path.push_back(syn2[i]);

			distance = path.size() - 1;
		}
	}

	string toString()  {
		stringstream ss(stringstream::in | stringstream::out);

		for (size_t i=0; i < path.size(); i++)  {
			for (size_t j=0; j < i; j++)
				ss << "  ";
			ss << "=> ";

			for (size_t j=0; j < path[i].wcount; j++)  {
				ss << path[i].words[j];

				if (j < path[i].wcount-1)
					ss << ", ";
			}

			ss << endl;
		}

		return ss.str();
	}
};

bool cmpPaths (CommonPath p1, CommonPath p2)  {
	return (p1.distance < p2.distance);
}

vector< vector<Synset> > getChain (string word, wn_rel rel)  {
	vector< vector<Synset> > syn;

	if (!wninit() && !morphinit()) {
		SynsetPtr syns;

		for (size_t i=NOUN; i <= SATELLITE; i++)  {
			for (syns = findtheinfo_ds(strdup(word.c_str()), i, -rel, ALLSENSES);
					syns; syns = syns->nextss)  {
				vector<Synset> s;

				for (SynsetPtr tmp_syns = syns; tmp_syns; tmp_syns = tmp_syns->ptrlist)  {
					s.push_back(*tmp_syns);
				}

				syn.push_back(s);
			}
		}
	}

	return syn;
}

vector<CommonPath> computeAffinities (char* lemma1, char* lemma2)  {
	vector<CommonPath> affinities;

	vector< vector<Synset> > syn1 = getChain(lemma1, hypernym);
	vector< vector<Synset> > syn2 = getChain(lemma2, hypernym);

	if (!syn1.size() || !syn2.size())
		return affinities;

	for (size_t i=0; i < syn1.size(); i++)  {
		for (size_t j=0; j < syn2.size(); j++)  {
			CommonPath path (syn1[i], syn2[j]);

			if (!path.path.empty())
				affinities.push_back(path);
		}
	}

	sort (affinities.begin(), affinities.end(), cmpPaths);
	return affinities;
}

int main (int argc, char **argv)  {
	if (!argv[1] || !argv[2])
		return 1;

	vector<CommonPath> affinities = computeAffinities(argv[1], argv[2]);
	vector<CommonPath>::iterator it;

	cout << "Affinities (class=>subclass) between " << argv[1] << " => " << argv[2] << "\n";
	
	for (it = affinities.begin(); it != affinities.end(); it++)  {

		for (size_t i=0; i < it->from().wcount; i++)  {
			cout << it->from().words[i];
			if (i < it->from().wcount-1)
				cout << ", ";
		}

		cout << "\tDistance: (" << it->distance << ")";
		cout << "\n\t=> ";

		for (size_t i=0; i < it->to().wcount; i++)  {
			cout << it->to().words[i];
			if (i < it->to().wcount-1)
				cout << ", ";
		}
		cout << "\n";
	}

	//cout << affinities[0].toString();
	return 0;
}


