/** @file    TelnetServerBundle.cpp
 *  @author  Alessandro Polo
 *  @version $Id: TelnetServerBundle.cpp 2732 2010-07-06 03:18:11Z alex $
 *  @brief
 * File containing methods for the wosh::services::TelnetServerBundle class.
 * The header for this class can be found in TelnetServerBundle.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "TelnetServerBundle.h"
 
 #include "TelnetServerImpl.h"

 #include <core/MessageFilter.h>
 #include <core/MethodsCommon.h>
 #include <core/MessageProcessingCommon.h>
 #include <core/Notification.h>
 #include <core/Fact.h>
 #include <core/Table.h>

 #include <interfaces/services/CommunicatorService.h>
 #include <framework/communication/SessionIM.h>
 #include <framework/communication/TextMessage.h>


using namespace std;
using namespace wosh::communication;
namespace wosh {
 namespace services {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

TelnetServerBundle::TelnetServerBundle()
	: BundleGeneric() {
	BundleGeneric::setName( "TelnetServer" );
	Properties.update( _KEY_Version, _TelnetServer_VERSION );

Log.setLevel( LOG_VERBOSE );

	Log(LOG_DEBUG, " Configuring TelnetServer worker.." );
	this->ssrvWorker = new TelnetServerImpl(*this);

	Log(LOG_DEBUG, " Setting default properties and permissions.." );
	Properties.update( _TelnetServer_KEY_WelcomeString, "Welcome WOSH TelnetServer v." _TelnetServer_VERSION );

	Log(LOG_DEBUG, " Registering methods.." );
	MethodMessageResponse* mmListSessions = Methods.create<MethodMessageResponse>( _METHOD_List, "List Sessions" );
	mmListSessions->setMethod( this, (MethodMessageResponsePtr)&TelnetServerBundle::mmDoListSessions );
	mmListSessions->getPermission().setMask( Permission::RX, Permission::RX, Permission::RX );

	setBundleState(Bundle::STATE_CREATED, false);
 }

TelnetServerBundle::~TelnetServerBundle() {
	Log(LOG_VERBOSE, " Destroying.." );
	if ( isBundleRunning() ) {
		Log(LOG_WARNING, "~TelnetServerBundle() : Destroying while RUNNING! Trying to stop.." );
		bundleStop();
	 }
	delete this->ssrvWorker; this->ssrvWorker = NULL;
	Log(LOG_INFO, ":~TelnetServerBundle() : Destroyed." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL

WRESULT TelnetServerBundle::bundleStart() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STARTING) ) return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::start_SynchThread( this->ssrvWorker );

	// BUNDLE-STATE (STARTED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

WRESULT TelnetServerBundle::bundleStop() {
	if ( !BundleGeneric::bundleValidate_StartStop(Bundle::STATE_STOPPING) ) return WRET_ERR_WRONG_STATE;

	WRESULT ret = BundleGeneric::stop_SynchThread( this->ssrvWorker );

	// BUNDLE-STATE (STOPPED) will be updated async by WORKER, through call: thread_event()
	return ret;
 }

////////////////////////////////////////////////////////////////////////////////////////////// BUNDLE CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES

void TelnetServerBundle::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;
	BundleGeneric::busMessage(message, source);
 }

//////////////////////////////////////////////////////////////////////////////////////// PROCESS BUS MESSAGES
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT

////////////////////////////////////////////////////////////////////////////////////////////// PROPERTY EVENT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetServerBundle::startSession( Session* session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;

	ret = this->ssrvWorker->registerSession( session );

	ret = session->sessionStart();

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT TelnetServerBundle::stopSession( Session* session )
 {
	if ( session == NULL ) return WRET_ERR_PARAM;
	WRESULT ret = WRET_OK;
	
	ret = session->sessionStop();
	
	ret = this->ssrvWorker->unRegisterSession( session );

	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
Session* TelnetServerBundle::newSession( const string& username )
 {
	Session* session = new Session();
	session->setUsername( "alex" );
	WRESULT ret = ssrvWorker->registerSession( session );
	Log(LOG_VERBOSE, ":newSession() : New Session for %s, id: %i", username.c_str(), session->getID() );
	return session;
 }
*/

bool TelnetServerBundle::session_is_auth( Session* session, const string& user, const string& pwd ) {
	return this->ssrvWorker->session_is_auth(session, user, pwd);
 }

bool TelnetServerBundle::session_post_message( Session* session, Message* message ) {
	return this->ssrvWorker->session_post_message(session, message);
 }

WRESULT TelnetServerBundle::changeSessionProtocol( Session* session, const string& protocol_name ) {
	return this->ssrvWorker->changeSessionProtocol(session, protocol_name);
 }
 

/*
Session* TelnetServerBundle::getSession( long id ) {
	return this->ssrvWorker->getSession(id);
 }

WRESULT TelnetServerBundle::registerSession( Session* session ) {
	return this->ssrvWorker->registerSession(session);
 }

WRESULT TelnetServerBundle::unRegisterSession( Session* session ) {
	return this->ssrvWorker->unRegisterSession(session);
 }
*/

bool TelnetServerBundle::isUserLogged( const std::string& username ) const {
	return this->ssrvWorker->isUserLogged(username);
 }

long TelnetServerBundle::getSessionIDByUser( const std::string& username ) const {
	return this->ssrvWorker->getSessionIDByUser(username);
 }

WRESULT TelnetServerBundle::registerProtocol( SessionProtocol* session_proto ) {
	return this->ssrvWorker->registerProtocol(session_proto);
 }

WRESULT TelnetServerBundle::unRegisterProtocol( SessionProtocol* session_proto ) {
	return this->ssrvWorker->unRegisterProtocol(session_proto);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

Response* TelnetServerBundle::mmDoListSessions( const Message* requestMessage )
 {
	if ( requestMessage == NULL || requestMessage->isEmpty() ) return NULL;
	if ( !requestMessage->getContent()->isRequest() ) return NULL;
	const Request* request = requestMessage->getContent()->asRequest();
	string option = request->getArgument().getData();
	Log(LOG_VERBOSE, ":mmDoListSessions(%s)", option.c_str() );

	if ( option == "s" || option == "short" ) {
		DataList* objs = new DataList();
		this->ssrvWorker->getSessions().mutex().lockForRead();
		std::vector<TelnetSession*>::const_iterator it;
		std::vector<TelnetSession*>::const_iterator it_end = this->ssrvWorker->getSessions().stdEnd();
		for ( it=this->ssrvWorker->getSessions().stdBegin(); it!=it_end; it++ ) {
			if ( *it == NULL ) continue;
			objs->add( new Data((*it)->getName()) );
		}
		this->ssrvWorker->getSessions().mutex().unLock();
		return new Response( request->getMethod(), objs );
	 }
	else {
		this->ssrvWorker->getSessions().mutex().lockForRead();
		Table* tbl = new Table( this->ssrvWorker->getSessions().size_(), 3, true );
		tbl->setTableName("TelnetSessions");
		tbl->setHeaderHorizontal( new Data("Name"), 0);
		tbl->setHeaderHorizontal( new Data("Username"), 1);
		tbl->setHeaderHorizontal( new Data("Perms"), 2);
		unsigned int iRow = 0;
		std::vector<TelnetSession*>::const_iterator it;
		std::vector<TelnetSession*>::const_iterator it_end = this->ssrvWorker->getSessions().stdEnd();
		for ( it=this->ssrvWorker->getSessions().stdBegin(); it!=it_end; it++ ) {
			if ( *it == NULL ) continue;
			const TelnetSession* tSess = *it;
			tbl->set( new Data(tSess->getName()), iRow, 0);
			tbl->set( new Data(tSess->getUsername()), iRow, 1);
			tbl->set( tSess->getPermission().clone(), iRow, 2);
			++iRow;
		}
		this->ssrvWorker->getSessions().mutex().unLock();
		return new Response( request->getMethod(), tbl );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace services
}; // namespace wosh
