/** @file    WeatherSourceIGoogleAPI.cpp
 *  @author  Alessandro Polo
 *  @version $Id: WeatherSourceIGoogleAPI.cpp 2804 2010-07-27 04:39:07Z alex $
 *  @brief
 * File containing methods for the WeatherSourceIGoogleAPI class.
 * The header for this class can be found in WeatherSourceIGoogleAPI.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "WeatherSourceIGoogleAPI.h"

 #include <core/Utilities.h>

 #include <QFile>
 #include <QtXml>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

WeatherSourceIGoogleAPI::WeatherSourceIGoogleAPI()
	: WeatherSourceHttp() {
	Object::setName("WeatherSourceIGoogleAPI");
	Log.setContext( "WeatherSourceIGoogleAPI" );
	this->priority = 30;
	this->reliability = 20.0;
	this->xmlUrl = "http://www.google.com/ig/api?weather=Rovereto&hl=en";
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherSourceIGoogleAPI::loadForecast( WeatherInfo* wCurrentInfo, std::vector<WeatherInfo*>* forecast ) {
	FilePath srcFile;
	srcFile.set( this->xmlLocalTemp );
	std::string filename = srcFile.toPathLocal();
	QDomDocument doc("none");
	QFile file(filename.c_str());
	if ( !file.open(QIODevice::ReadOnly) ) {
		Log(LOG_CRITICAL, ":loadForecast() FAILED : Can't open file: %s", filename.c_str() );
		return WRET_ERR_PARAM;
	 }
	if ( !doc.setContent(&file) ) {
		file.close();
		Log(LOG_CRITICAL, ":loadForecast() FAILED : Can't set contents from: %s", filename.c_str() );
		return WRET_ERR_PARAM;
	 }
	file.close();
	QDomElement api_reply_Node = doc.documentElement();
	if ( api_reply_Node.isNull() ) {
		Log(LOG_WARNING, ":loadForecast() FAILED : No 'xml_api_reply' node in %s", filename.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	QDomElement weatherNode = api_reply_Node.namedItem("weather").toElement();
	if ( weatherNode.isNull() ) {
		Log(LOG_WARNING, ":loadForecast() FAILED : No 'weather' node in %s", filename.c_str() );
		return WRET_ERR_INTERNAL;
	 }
	int64 dataTimestamp = FileSystem::getTimeStampOf(filename);
	if ( wCurrentInfo != NULL ) {
/*
<weather module_id="0" tab_id="0" mobile_row="0" mobile_zipped="1" row="0" section="0">
  <forecast_information>
    <city data="Rovereto, Trentino-Alto Adige/Sdtirol"/>
    <postal_code data="Rovereto"/>
    <latitude_e6 data=""/>
    <longitude_e6 data=""/>
    <forecast_date data="2010-07-26"/>
    <current_date_time data="2010-07-26 13:38:58 +0000"/>
    <unit_system data="US"/>
  </forecast_information>
  <current_conditions>
    <condition data="Partly Cloudy"/>
    <temp_f data="81"/>
    <temp_c data="27"/>
    <humidity data="Humidity: 37%"/>
    <icon data="/ig/images/weather/partly_cloudy.gif"/>
    <wind_condition data="Wind: SW at 6 mph"/>
  </current_conditions>
  <forecast_conditions>
    <day_of_week data="Mon"/>
    <low data="67"/>
    <high data="78"/>
    <icon data="/ig/images/weather/rain.gif"/>
    <condition data="Rain"/>
  </forecast_conditions>
</weather>
*/
		QDomElement infoNode = weatherNode.namedItem("forecast_information").toElement();
		if ( !infoNode.isNull() ) {

			QDomElement cityNode = infoNode.namedItem("city").toElement();
			if ( !cityNode.isNull() && cityNode.hasAttribute("data") )
				this->city = cityNode.attribute("data").toLatin1().data();
			QDomElement postalNode = infoNode.namedItem("postal_code").toElement();
			if ( !postalNode.isNull() && postalNode.hasAttribute("data") )
				this->city_postal = postalNode.attribute("data").toLatin1().data();

			QDomElement dtNode = infoNode.namedItem("current_date_time").toElement();
			if ( !dtNode.isNull() && dtNode.hasAttribute("data") ) {
				QDateTime dt = QDateTime::fromString(dtNode.attribute("data"), Qt::ISODate );
				dataTimestamp = dt.toTime_t();
			 }
			QDomElement fcdtNode = infoNode.namedItem("forecast_date").toElement();
			if ( !fcdtNode.isNull() && fcdtNode.hasAttribute("data") ) {
				QDateTime dt = QDateTime::fromString(fcdtNode.attribute("data"), Qt::ISODate );
				wCurrentInfo->setPeriodDay(  dt.toTime_t() );
				//wCurrentInfo->setPeriodName("");
			 }
 		 }

		QDomElement currConditionNode = weatherNode.namedItem("current_conditions").toElement();
		if ( !currConditionNode.isNull() ) {
			Log(LOG_INFO, ":loadForecast() : Parsing Current conditions.." );
			configureWeatherInfo(wCurrentInfo, dataTimestamp);

			QDomElement TempNode = currConditionNode.namedItem("temp_c").toElement();
			if ( !TempNode.isNull() && TempNode.hasAttribute("data") )
				wCurrentInfo->setTemperature( TempNode.attribute("data").toInt() );
			QDomElement humidityNode = currConditionNode.namedItem("humidity").toElement();
			if ( !humidityNode.isNull() && humidityNode.hasAttribute("data") )
				wCurrentInfo->setAtmosphereHumidity( humidityNode.attribute("data").right(3).left(2).toInt() );
			QDomElement conditionNode = currConditionNode.namedItem("condition").toElement();
			if ( !conditionNode.isNull() && conditionNode.hasAttribute("data") ) {
				std::string c_str = conditionNode.attribute("data").toLatin1().data();
				int condition, xtra; condition = xtra = 0;
				parseWeatherCondition(c_str, condition, xtra);
				if ( condition == 0 )
					Log(LOG_WARNING, ":loadForecast() Unknown condition '%s'", c_str.c_str() );
				wCurrentInfo->setCondition(condition, xtra);
				wCurrentInfo->setConditionKey( c_str );
				wCurrentInfo->setConditionSummary( c_str );
			 }
		}
		else {
			Log(LOG_WARNING, ":loadForecast() FAILED : No 'current_conditions' node in %s", filename.c_str() );
		 }
	 }

	if ( forecast != NULL ) {
		Log(LOG_INFO, ":loadForecast() : Parsing Forecast conditions.." );
		QDomNodeList forecastList = weatherNode.elementsByTagName( "forecast_conditions" );
		for(int j=0; j<forecastList.size(); j++) {
			QDomElement cForecastNode = forecastList.item(j).toElement();
			WeatherInfo* wInfo = new WeatherInfo();
			configureWeatherInfo(wInfo, dataTimestamp);
			QDomElement dayOfWeekNode = cForecastNode.namedItem("day_of_week").toElement();
			if ( !dayOfWeekNode.isNull() && dayOfWeekNode.hasAttribute("data") )
				wInfo->setPeriodName( dayOfWeekNode.attribute("data").toLatin1().data() );
			QDomElement TempLowNode = cForecastNode.namedItem("low").toElement();
			if ( !TempLowNode.isNull() && TempLowNode.hasAttribute("data") )
				wInfo->setTemperatureLowest( TempLowNode.attribute("data").toInt() );
			QDomElement TempHighNode = cForecastNode.namedItem("high").toElement();
			if ( !TempHighNode.isNull() && TempHighNode.hasAttribute("data") )
				wInfo->setTemperatureHighest( TempHighNode.attribute("data").toInt() );
			QDomElement conditionNode = cForecastNode.namedItem("condition").toElement();
			if ( !conditionNode.isNull() && conditionNode.hasAttribute("data") ) {
				std::string c_str = conditionNode.attribute("data").toLatin1().data();
				int condition, xtra; condition = xtra = 0;
				parseWeatherCondition(c_str, condition, xtra);
				if ( condition == 0 )
					Log(LOG_WARNING, ":loadForecast() Unknown condition '%s'", c_str.c_str() );
				wInfo->setCondition(condition, xtra);
				wInfo->setConditionKey( c_str );
				wInfo->setConditionSummary( c_str );
			 }
			forecast->push_back(wInfo);
		 }
	 }
	return WRET_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT WeatherSourceIGoogleAPI::parseWeatherCondition( const std::string& condition_str, int& condition, int& xtra ) const {
	std::string lcond = condition_str;
	Utilities::toLower(lcond);

	if ( lcond.find("cloudy") != std::string::npos )
		condition |= (int)WeatherInfo::CONDITION_CLOUDY;
	if ( lcond.find("sunny") != std::string::npos )
		condition |= (int)WeatherInfo::CONDITION_SUNNY;
	if ( lcond.find("rain") != std::string::npos )
		condition |= (int)WeatherInfo::CONDITION_RAIN;
	if ( lcond.find("showers") != std::string::npos ) {
		condition |= (int)WeatherInfo::CONDITION_RAIN;
		xtra |= (int)WeatherInfo::CONDITION_XTRA_SCATTERED;
	 }

//	if ( lcond.find("windy") != std::string::npos )
//		condition |= (int)WeatherInfo::CONDITION_UNKNOWN;

	if ( lcond.find("partly") != std::string::npos )
		xtra |= (int)WeatherInfo::CONDITION_XTRA_PARTLY;


	return WRET_OK;
 }