/**
 * @class   wosh::BundleRequirements
 * @brief   
 *          
 *          
 *          
 ****************************************************************************
 * @version $Id: BundleRequirements.h 2555 2010-05-26 22:25:28Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_BundleRequirements_H__
 #define __WOSH_Core_BundleRequirements_H__

 #include <woshDefs.h>
 #include <core/Serialization.h>
 #include <core/Containers.h>
 #include <core/Bundle.h>
 #include <core/ObjectAllocator.h>

 
namespace wosh {


// subject requires this item to be validated
class BundleRequirement
 {
	public:
		enum REQUIREMENT_TYPE {
			TYPE_UNKNOWN		= 0x00,
			TYPE_BUNDLE			= 0x01,	///< 
			TYPE_OBJECT			= 0x02,	///< 
			TYPE_FRAMEWORK		= 0x04	///< 
		};
		enum REQUIREMENT_EFFECT {
			EFFECT_UNKNOWN		= 0x00,
			EFFECT_DELAY_RETRY	= 0x01,	///< on failure, delay current routine and retry (once)
			EFFECT_FIX			= 0x02,	///< on failure, try to fix the scope (load required bundle, start it, ..)
			EFFECT_FAIL			= 0x08	///< on failure, fail immediatly
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		BundleRequirement()
			: classInfo(), locationHopsMax(-1), reqScope(Bundle::STATE_UNKNOWN) { }
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		BundleRequirement( const BundleRequirement& m ) {
			*this = m;
		 }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~BundleRequirement() { }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		const ObjectTypeInfo& getClassInfo() const		{ return this->classInfo; }

		const std::string& getClassType() const			{ return this->classInfo.name; }
		double getClassVersionMin() const				{ return this->classInfo.version; }
		int getLocationHopsMax() const					{ return this->locationHopsMax; }
		int getScopes() const							{ return this->reqScope; }
//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setClassType( const std::string& value )	{ this->classInfo.name = value; }
		void setClassVersionMin( double value )			{ this->classInfo.version = value; }
		void setLocationHopsMax( int value )			{ this->locationHopsMax = value; }
		void setScopes( int value )						{ this->reqScope = value; }
//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		wosh::BundleRequirement& operator=(const wosh::BundleRequirement& m) {
			if (this == &m) return *this; // same object?
			this->reqScope = m.reqScope;
			this->classInfo = m.classInfo;
			this->locationHopsMax = m.locationHopsMax;
			return *this;
		 }
		bool operator==(const wosh::BundleRequirement &other) const {
			if ( this->classInfo != other.classInfo ) return false;
			if ( this->reqScope != other.reqScope ) return false;
			if ( this->locationHopsMax != other.locationHopsMax ) return false;
			return true;
		 }
		bool operator!=(const wosh::BundleRequirement &other) const { return !(*this == other);  }
		// operator &

//@}
	protected: // target identification
		ObjectTypeInfo classInfo;
		int locationHopsMax;			///< -1 = ANY
		int reqScope;

}; // class def




class List;
class Table;

class BundleRequirements : public ISerializable
 {
	WOSH_SERIALIZABLE(wosh::BundleRequirements)
	WOSH_COMPARE_RTTI(wosh::BundleRequirements)

	typedef ListT<BundleRequirement*> tBundleRequirementList;


	public:
		/**
		 * @brief  Empty constructor.
		 */
		BundleRequirements() { }
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		BundleRequirements( const BundleRequirements& m ) {
			*this = m;
		 }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~BundleRequirements();

	public:
		virtual WRESULT add( BundleRequirement* requirement );

		WRESULT add( const std::string& classType, Bundle::BUNDLE_STATE scope = Bundle::STATE_STARTING, double interfaceVersionMin = 0.0 );


		template <class OBJECT_TYPE>
		WRESULT add( Bundle::BUNDLE_STATE scope = Bundle::STATE_STARTING, double interfaceVersionMin = 0.0 ) {
			return add(OBJECT_TYPE::className(), scope, interfaceVersionMin );
		 }


	public:
		List* toList() const;
		Table* toTable() const;

		
	
		inline int count() const								{ return this->requirements.count(); }
		int count( Bundle::BUNDLE_STATE scope ) const;

		// get copy 
		int getRequirementsOf( std::vector<BundleRequirement*>& req_list, Bundle::BUNDLE_STATE scope ) const;

		const tBundleRequirementList& getRequirements() const	{ return this->requirements; }
		tBundleRequirementList& getRequirements()				{ return this->requirements; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		wosh::BundleRequirements& operator=(const wosh::BundleRequirements& m);
		bool operator==(const wosh::BundleRequirements &other) const;
		bool operator!=(const wosh::BundleRequirements &other) const { return !(*this == other);  }
		bool operator>(const wosh::BundleRequirements& other) const;
		bool operator<(const wosh::BundleRequirements& other) const;
//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		tBundleRequirementList requirements;

}; // class def

}; // namespace wosh

#endif //__WOSH_Core_BundleRequirements_H__
