/** @file    BusConnector.cpp
 *  @author  Alessandro Polo
 *  @version $Id: BusConnector.cpp 2370 2010-04-27 13:55:58Z alex $
 *  @brief
 * File containing methods for the wosh::BusConnector class.
 * The header for this class can be found in BusConnector.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/BusConnector.h>

 #include <core/ObjectFactory.h>
 #include <core/BusManager.h>
 #include <core/WoshKernel.h>

#ifdef _ENABLE_PROFILING
 #include <core/SystemProfiler.h>
#endif


using namespace std;
namespace wosh {

 WOSH_REGISTER(wosh::BusConnector, "", 1.00, _static_BusConnector )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

BusConnector::BusConnector() {
	this->bus = NULL;
	this->handler = NULL;
 }
 
BusConnector::~BusConnector() {
	disconnect();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// (DIS)CONNECTION

WRESULT BusConnector::connect( const std::string& busname, bool create_if_required ) {
	if ( this->bus != NULL ) return WRET_ERR_ILLEGAL_USE;
	if ( WoshKernel::getInstance() == NULL ) return WRET_ERR_INTERNAL;

	this->bus = WoshKernel::getInstance()->busses().connectBus( this, busname, create_if_required );
	if ( this->bus == NULL )
		return WRET_ERR_INTERNAL;
	if ( this->handler != NULL ) { // init again to fix busname
		PROFILING_INIT( this, this->handler )
	 }
	return WRET_OK;
 }

WRESULT BusConnector::disconnect() {
	if ( this->bus == NULL ) return WRET_ERR_ILLEGAL_USE;
	WRESULT ret = bus->removeBusListener(this);
	this->bus = NULL;
	if ( this->handler != NULL ) { // init again to fix busname
		PROFILING_INIT( this, this->handler )
	 }
	return ret;
 }

///////////////////////////////////////////////////////////////////////////////////////////// (DIS)CONNECTION
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////// GETTERS/SETTERS

bool BusConnector::isConnected() const {
	if ( this->bus == NULL )
		return false;
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string BusConnector::getBusName() const {
	if ( this->bus == NULL )
		return "";
	return this->bus->getName();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BusConnector::setMessageHandler( IBusListener* Handler ) {
	this->handler = Handler;
	PROFILING_INIT( this, this->handler );
 }

///////////////////////////////////////////////////////////////////////////////////////////// GETTERS/SETTERS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS

void BusConnector::busEvent( Bus::BUS_STATE state, const Bus* source )
 {
	 if ( state == Bus::STATE_DESTROYED )
		this->bus = NULL;
	if ( this->handler == NULL )
		return;
	this->handler->busEvent( state, source );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BusConnector::busMessage( const Message& message, const Bus* source ) {
	if ( this->handler == NULL ) return;
	PROFILING_ENTER(this, 0)
	this->handler->busMessage( message, source );
	PROFILING_EXIT(this, 0)
 }

////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// POSTING

bool BusConnector::postMessage( Message* message ) const {
	if ( this->bus == NULL ) return false;
	PROFILING_ENTER(this, 1)
	this->bus->postMessage( message );
	PROFILING_EXIT(this, 1)
	return true;
 }

///////////////////////////////////////////////////////////////////////////////////////////////////// POSTING
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
Message* BusConnector::postMessageWaitReply( Message* message, long timeout )
 {
	if ( this->bus == NULL ) return NULL;

	Message* reply = NULL;

	return reply;
 }
*/
///////////////////////////////////////////////////////////////////////////////////////////////////// POSTING
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
