/**
 * @class   wosh::Communicator
 * @brief   The class holds information about a generic communicator service,
 *          defined by wosh::interfaces::services::CommunicatorService
 *
 * @see     wosh::services::CommunicatorManagerBundle
 ****************************************************************************
 * @version $Id: Communicator.h 2676 2010-06-20 02:42:08Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Communicator_H__
 #define __WOSH_Core_Communicator_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/Containers.h>
 #include <core/SharedObject.h>
 #include <core/InterfacesProvider.h>
 #include <core/Notification.h>
 #include <core/Utilities.h>
 #include <core/DataModels.h>


namespace wosh {

class Communicator :	public Object,
						public SharedObject
 {
	WOSH_SERIALIZABLE(wosh::Communicator)
	WOSH_COMPARE_RTTI(wosh::Communicator)

	public:

		enum CHANNEL_LINK {
			LINK_UNKNOWN = 0,
			LINK_SYNCH = 1,
			LINK_ASYNCH = 2
		};

		enum CHANNEL_DUPLEX {
			DUPLEX_UNKNOWN = 0,
			DUPLEX_OUT = 1,
			DUPLEX_IN = 2,
			DUPLEX_IO = 3
		};


	public:
		/**
		 * @brief  Empty constructor.
		 */
		Communicator() : Object(), SharedObject() {
			Object::setName( "Communicator" );
			this->timestamp = 0;
			this->channelTypes = Notification::TYPE_UNKNOWN;
			this->channelLink = Communicator::LINK_UNKNOWN;
			this->protocol = Notification::PROTOCOL_UNKNOWN;
			this->cost = 0;
			this->priority = 0;
			this->reliability = 0;
			setSeen();
		 }
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Communicator( const wosh::Communicator& m ) : Object(), SharedObject(m) {
			*this = m;
		 }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Communicator() { }


	public:
		inline const std::string& getSharedID() const		{ return Object::getEntityID(); }


/** @name Getters
 * @{
 ******************************************************************************/
	public:
		int getChannelTypes() const							{ return this->channelTypes; }
		std::string getChannelTypesAsString() const			{ return getChannelTypesAsString(this->channelTypes); }

		CHANNEL_LINK getChannelLink() const					{ return this->channelLink; }

		Notification::PROTOCOL getProtocol() const			{ return this->protocol; }
		inline const char* getProtocolAsString() const		{ return Notification::getProtocolAsString(this->protocol); }

		double getCost() const								{ return this->cost; }
		int getPriority() const								{ return this->priority; }
		int getReliability() const							{ return this->reliability; }

		const InterfacesProvider& getInterfaces() const		{ return this->Interfaces; }
		InterfacesProvider& getInterfaces()					{ return this->Interfaces; }

		long getSeen() const								{ return this->timestamp; }

		const std::vector<std::string>& getLocations() const { return this->locations; }
		std::vector<std::string>& getLocations()			{ return this->locations; }

		bool isInLocation( const std::string& location ) const {
			std::vector<std::string>::const_iterator it;
			for ( it=this->locations.begin(); it!=this->locations.end(); ++it ) {
				if ( *it != location ) continue;
				return true;
			 }
			return false;
		 }

		static std::string getChannelTypesAsString( int types ) {
			std::vector<std::string> typesStr;
			if ( types & (int)Notification::TYPE_TEXTUAL )
				typesStr.push_back(Notification::getNotificationTypeAsString(Notification::TYPE_TEXTUAL));
			if ( types & (int)Notification::TYPE_DATA )
				typesStr.push_back(Notification::getNotificationTypeAsString(Notification::TYPE_DATA));
			if ( types & (int)Notification::TYPE_AUDIO )
				typesStr.push_back(Notification::getNotificationTypeAsString(Notification::TYPE_AUDIO));
			if ( types & (int)Notification::TYPE_VIDEO )
				typesStr.push_back(Notification::getNotificationTypeAsString(Notification::TYPE_VIDEO));
			return Utilities::joinVector(typesStr, "|");
		 }


		static DataFieldCollector<Communicator>* getDataFields() {
			DataFieldCollector<Communicator>* fields = new DataFieldCollector<Communicator>();
			fields->add<const URI&,URI>(&Communicator::getURI, "URI", 0);
			fields->add<long,Variant_DT>(&Communicator::getSeen, "SeenAt", 1);
			fields->add<double,Variant>(&Communicator::getCost, "Cost", 2);
			fields->add<int,Variant>(&Communicator::getReliability, "Reliability", 3);
			fields->add<CHANNEL_LINK,Variant>(&Communicator::getChannelLink, "ChLink", 4);
			fields->add<int,Variant>(&Communicator::getChannelTypes, "ChType", 5);
			return fields;
		 }

//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		virtual void setName( const std::string& value ) {
			Object::setName(value);
			bool glob = Object::getURI().isGlobal();
			if ( glob )
				Object::getURI().unRegisterGlobal();
			Object::getURI().setName( Object::getName() );
			if ( glob )
				Object::getURI().registerGlobal();
		 }

		using Object::setURI;

		void setChannelTypes( int value )					{ this->channelTypes = value; }
		void setChannelLink( CHANNEL_LINK value )			{ this->channelLink = value; }

		void setProtocol( Notification::PROTOCOL value )	{ this->protocol = value; }
		

		void setCost( double value )						{ this->cost = value; }
		void setPriority( int value )						{ this->priority = value; }
		void setReliability( int value )					{ this->reliability = value; }

		void setSeen()										{ this->timestamp = Utilities::std_time(); }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:

		virtual Communicator& operator=(const Communicator& m) {
			if (this == &m) return *this; // same object?
			(Object&)*this = (const Object&)m;
			this->timestamp = m.timestamp;
			this->Interfaces = m.Interfaces;
			this->channelTypes = m.channelTypes;
			this->channelLink = m.channelLink;
			this->protocol = m.protocol;
			this->cost = m.cost;
			this->priority = m.priority;
			this->reliability = m.reliability;
			this->locations = m.locations;
			return *this;
		 }

		virtual bool operator==(const Communicator &other) const {
			if ( this->Interfaces != other.Interfaces )	return false;
			if ( this->channelTypes != other.channelTypes ) return false;
			if ( this->channelLink != other.channelLink ) return false;
			if ( this->protocol != other.protocol )	return false;
			return false;
		 }

		virtual bool operator!=(const Communicator &other) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const	{ return ""; }
		virtual bool updateHash( DataHashContext* hash_context ) const {
			if ( hash_context == NULL ) return false;
			bool ret = Object::updateHash( hash_context );
			ret = hash_context->update( this->timestamp ) & ret;
			ret = this->Interfaces.updateHash( hash_context ) & ret;
			ret = hash_context->update( this->channelTypes ) & ret;
			ret = hash_context->update( this->channelLink ) & ret;
			ret = hash_context->update( this->protocol ) & ret;
			ret = hash_context->update( this->cost ) & ret;
			ret = hash_context->update( this->priority ) & ret;
			ret = hash_context->update( this->reliability ) & ret;
			ret = hash_context->update( this->locations ) & ret;
			return ret;
		 }
//@}
	protected:
		virtual void updated( const SharedObject* )  { }

	protected:
		long timestamp;						///< last seen (epoch) of real object
		InterfacesProvider Interfaces;		///< implemented interfaces

		int channelTypes;					///< describes the supported service {as messaging, phone, video}. See CHANNEL_TYPE
		CHANNEL_LINK channelLink;			///< describes the communication channel {synch, asynch}

		Notification::PROTOCOL protocol;	///< describes the communication protocol {human language, shell syntax, ..}
		double cost;						///< cost of the service,   / message
		int priority;						///< priority of the service
		int reliability;					///< reliability of the service

		std::vector<std::string> locations;	///< location(s) of communicator (usually its a device)


}; // class def


struct cmpCommunicatorPriority {
    bool operator()( Communicator* c1, Communicator* c2 ) {return c1->getPriority() < c2->getPriority();}
};
struct cmpCommunicatorReliability {
    bool operator()( Communicator* c1, Communicator* c2 ) {return c1->getReliability() < c2->getReliability();}
};


}; // namespace wosh

#endif //__WOSH_Core_Communicator_H__
