/**
 * @file    DataMining.h
 * @brief   Under PLANNING.
 *
 *          Data Mining is one requirement of applied Artificial Intelligence in WOSH.
 *          See also \ref page_ai.
 *
 * Related Topics:
 *  - prediction, relationship, description
 *  - natural language
 *
 *
 * Given a generic DataMiningProvider object, we know:
 *   - implemented interfaces (\c IsA relation)
 *   - offered (common) fields (\c HasA relation)
 *
 * a class names, hierarchical until core (and well-known interfaces) types.
 *
 * Example:
 *  wosh::building::Garden \b "is a" {wosh::building::Garden, wosh::building::Floor, wosh::building::BuildingObject, wosh::Object}
 *  wosh::building::Garden \b "has a" {wosh::PropertiesProvider, wosh::MethodsProvider}
 *
 *
 *
A framework for use with object-oriented programming systems provides a reusable object oriented (OO) framework for use with object oriented programming systems that provides an information retrieval (IR) shell that permits a framework user to define an index class that includes word index objects and provides an extensible information retrieval system that evaluates a user query by comparing information contained in the user query with information contained in the word index objects that relates to stored documents. The information in word index objects is produced by preprocessing operations on documents such that the documents relevant to the user query will be identified, thereby providing a query result. The information retrieval system user can load documents into the computer system storage, index documents so their information can be subject to a query search, and request query evaluation to identify and retrieve documents most closely related to the subject matter of a user query.

Knowledge representation, a subtopic of AI, studies how knowledge is organized and processed, what kind of data structures an intelligent agent uses and what kind of reasoning can and cannot be done with the knowledge
Knowledge representation has a vital role to play in AI, because it determines to a very large extent what kind of reasoning can be done with the knowledge, how fast it is, how much memory is consumed and how optimal and complete the algorithms that utilize the knowledge are.
Pros of object-oriented knowledge representation:
 - The analogy between software models and physical models.
 - The resilience of the software models.
 - The reusability of the components of the software models



 *
 *data warehousing
 *
 * References: Pesonen.-.Concepts.and.Object-Oriented.Knowledge.Representation.(2002).pdf
 
 *
 ****************************************************************************
 * @version $Id: DataMining.h 2596 2010-05-28 23:18:30Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_DataMining_H__
 #define __WOSH_Core_DataMining_H__

 #include <woshDefs.h>
 #include <core/Serializable.h>
 #include <vector>


namespace wosh {


class DataMeta
 {
	public:
		DataMeta() { }
		virtual ~DataMeta() { }

	public: // fields:
		// ....

		//ListT<std::string> keywords;
		// string isKindOf;

}; // class def

/*
IDEA:
  - object (dataminer) to be used as methodprovider, propertiesprovider, ..
  - architecture similar to object allocator/info/factory
*/

class DataMine : public DataMeta
 {

	public:
		DataMine()
			: DataMeta() { }
		virtual ~DataMine() { }

	public:


	protected:
		void registerMySelf();
		void unRegisterMySelf();


}; // class def


class DataMiner
 {
	public:
		DataMiner()
			: owner(NULL) { }
		virtual ~DataMiner() { }

	public:
		void rebind( IReflection* object ) {
			this->owner = object;
			// notify update
		 }
		void unBind() {
			this->owner = NULL;
			// retrieve class inheritance/poly from ObjectFactory
			// notify update
		 }

	public:
		void registerChild( const IReflection* object, bool notify_update = true ) {
			this->childs.transactionBeginWrite();
			this->childs.push_back(object);
			if ( notify_update ) {
				// notify update
			 }
			this->childs.transactionEnd();
		 }

		void unRegisterChild( const IReflection* object, bool notify_update = true ) {
			this->childs.transactionBeginWrite();
			this->childs.pop(object);
			if ( notify_update ) {
				// notify update
			 }
			this->childs.transactionEnd();
		 }

	public:
		IReflection* owner;
		ListT<IReflection*> childs;

}; // class def


class myMinedObject : public Object
 {
	WOSH_CLASSNAME(wosh::myMinedObject)
	WOSH_CLASS_ABSTRACT(false)

	public:
		myMinedObject()
			: Object() {
			
			Miner.rebind(this);
			Miner.registerChild(&Properties, false);
			Miner.registerChild(&Methods);
			//Miner.mine();

		 }
		virtual ~myMinedObject() {
		
			Miner.unBind();
		 }

	protected:
		PropertiesProvider Properties;
		MethodsProvider Methods;

		DataMiner Miner;
};



/*
//------------------------------------------------------------------------ 
A better interpretation of the relationship in question [inheritance] is that "birds
typically have wings," or "birds usually have wings." But, then, what is the
precise meaning of typically or of usually, []?
The simplest possible option is to force the classes to include a query mechanism
that returns the typicality of an instance, say, in the range of [0,1]. So, if every object has a
method called something like getTypicality() we can query their goodness and, for
example, sort the objects using their typicality values.


//------------------------------------------------------------------------ 
An intelligent being cannot treat every object it sees as a unique entity unlike
anything else in the universe. It has to put objects in categories so that it may
apply its hard-won knowledge about similar objects, encountered in the past, to
the object at hand. (Pinker 1997, 12).

WOSH_MINING_CATEGORY( CLASSNAME, CATEGORY, temp_static_name )


//------------------------------------------------------------------------ 
The similarity of an instance is calculated by checking how much it has in common
with the prototype, how many features are distinct to the prototype and how many features
are distinct to the instance. In terms of set theory this is equivalent of subtracting from the
intersection of P and I, the difference of P and I and the difference of I and P, where P is
the prototype and I is an instance. Formally it looks like this:
Sim(P,I) = af(P & I) - bf(P - I) - cf(I - P)
where a, b and c are factors that weight the importance of each set.


//------------------------------------------------------------------------ 




//------------------------------------------------------------------------ 



//------------------------------------------------------------------------ 






*/


//------------------------------------------------------------------------ 
// IDEA: classification:
/*
tree of documents, a node may be child of multiple parents at same time
 
 - WOSH_STATIC_KNOWLEDGE
	is pre-compiled or embedded into built
	=> soft clustering
	=> based on RTTI information (superclasses)

 - WOSH_DYNAMIC_KNOWLEDGE
	is created/updated on system initialization/some interrupts and periodically


 - WOSH_REALTIME_KNOWLEDGE
	is evaluated in REAL-TIME using ascynch message I/O, entities should refer to DYNAMIC KNOWLEDGE

-----------------
sample:

 - WOSH_STATIC_KNOWLEDGE
    - enternainment
      - audio
        - media
          - CLASS wosh::enternainment::MediaTrack
        - player
          - CLASS wosh::devices::PlayerGStreamer
          - ..
    - ..
 - WOSH_DYNAMIC_KNOWLEDGE
    - ..
    - enternainment
      - audio
        - media
        - player
          - INSTANCE PlayerHouse [wosh::devices::PlayerGStreamer]
    - ..
 - WOSH_REALTIME_KNOWLEDGE
    - ..
    - enternainment
      - audio
        - media
        - player
          - STATE/QUERY PlayerHouse [wosh::devices::PlayerGStreamer]
    - ..






*/
//------------------------------------------------------------------------ 
// old idea:




// many stuff should be provided by IReflection


class IDataMiner
 {

/** @name Data-Mining Generalization interface (IsA)
 * @{
 ******************************************************************************/
	public:
		// => implemented using ObjectFactory vs. IReflection class graph
			//	virtual inline bool isA( const char* classname ) const {
			//		if ( classname == NULL ) return false;
			//		return false;
			//	 }

//@}
/** @name Data-Mining Composition interface (HasA)
 * @{
 ******************************************************************************/
	public:
		// example: has(Properties) ?
		virtual bool hasComponent( const char* classname ) const = 0;

		//virtual void getComponentMap( const std::map<std::string, std::string> ) const = 0;

		virtual const IReflection* getComponent( const char* classname ) const = 0;

		template <typename T>
		inline const T* getComponentAs( const char* classname ) const {
			return dynamic_cast<const T*>(getA(classname));
		 }

//@}



	public:
		virtual ~DataMiningProvider() { }


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_DataMining_H__
