/**
 * @class   wosh::Device
 * @brief   The class holds information about a generic device,
 *          defined by wosh::interfaces::Device
 *
 ****************************************************************************
 * @version $Id: Device.h 2676 2010-06-20 02:42:08Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Device_H__
 #define __WOSH_Core_Device_H__

 #include <woshDefs.h>
 #include <core/Object.h>
 #include <core/InterfacesProvider.h>
 #include <core/SharedObject.h>

namespace wosh {

 #define _Device_KEY_DeviceStatus		"DeviceStatus"

 #define _Device_EVENT_UserInteraction	"event_user_interaction"

 #define _Device_EVENT_StateChanged		"StateChanged"


class Device :	public Object,
				public SharedObject
 {
	WOSH_SERIALIZABLE(wosh::Device)
	WOSH_COMPARE_RTTI(wosh::Device) // avoid dominance inheritance warning


	public:
		/**
		 * @brief  Empty constructor.
		 */
		Device() : Object(), SharedObject() {
			Object::setName( "Device" );
		 }
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Device( const Device& m ) : Object(m), SharedObject(m) {
			this->bundleOwner = m.bundleOwner;
		 }

		Device( const Object& parent ) : Object(parent) {
			this->bundleOwner = parent.getName();
		 }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Device() { }

	public:
		inline const std::string& getSharedID() const		{ return Object::getEntityID(); }

		virtual void setName( const std::string& value ) {
			Object::setName(value);
			bool glob = Object::getURI().isGlobal();
			if ( glob )
				Object::getURI().unRegisterGlobal();
			Object::getURI().setName( Object::getName() );
			if ( glob )
				Object::getURI().registerGlobal();
		 }

		const std::string& getBundleOwner() const			{ return this->bundleOwner; }
		std::string& getBundleOwner()						{ return this->bundleOwner; }

		const InterfacesProvider& getInterfaces() const		{ return this->Interfaces; }
		InterfacesProvider& getInterfaces()					{ return this->Interfaces; }

		void setBundleOwner( const std::string& value )		{ this->bundleOwner = value; }

		//state: on, off, unknown

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const	{ return ""; }
		virtual bool updateHash( DataHashContext* hash_context ) const {
			if ( hash_context == NULL ) return false;
			bool ret = Object::updateHash( hash_context );
			ret = hash_context->update( this->bundleOwner ) & ret;
			ret = this->Interfaces.updateHash( hash_context ) & ret;
			return ret;
		 }

//@}
	protected:
		std::string bundleOwner;
		InterfacesProvider Interfaces;


}; // class def

}; // namespace wosh

#endif //__WOSH_Core_Device_H__
