/** @file    Session.cpp
 *  @author  Alessandro Polo
 *  @version $Id: Session.cpp 2804 2010-07-27 04:39:07Z alex $
 *  @brief
 * File containing methods for the wosh::communication::Session class.
 * The header for this class can be found in Session.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/Session.h>

 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>
 #include <core/DataModels.h>
 #include <core/MessageFilter.h>
 #include <core/SessionManager.h>
 #include <core/SecurityManager.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER_INTERFACE(wosh::Session, "wosh::Object", 1.0, _static_Session )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

Session::Session()
	: Object() {
	this->id = Utilities::randomInt();
	this->id = _GET_Abs(this->id);
	Object::setName("Session", this->id);
	Object::getURI().setKernelLocal();
	Object::getURI().setPath(_SessionManager_URI);
	Object::getURI().setName( Utilities::toString(this->id) );

	Log.setContext( "Session", true );
	Log.setIndent( 3 );
	Log.setLevel( LOG_INFO );
Log.setLevel( LOG_VERBOSE );

	this->protocol = NULL;
	this->credentials = NULL;
	this->state = Session::STATE_IDLE;

	this->username = "";
	this->usernameLogged = "";
	this->msgWelcome = "";
	this->skipWelcome = false;
	this->currentBus = _Bus_ANY;

	this->creation_ts = Utilities::std_time();
//	this->inputTimeout = 0;
//	this->initDelay = 0;
	this->lastInputTS = 0;
	this->historyMaxSize = 20;
	this->currentLocation.setKernelLocal();

 }

Session::~Session() {
	this->state = Session::STATE_CLOSED;

 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Session::login( const SessionInfo* session_info ) {
	if ( session_info == NULL ) return WRET_ERR_PARAM;
	session_info->transactionBeginRead();
	WRESULT ret = login( session_info->getUserName(), session_info->getCredentials() );
	session_info->transactionEnd();
	return ret;
 }

WRESULT Session::login( const std::string& wosh_username, const SecurityTokenProvider* new_credentials ) {
	this->state = Session::STATE_LOGGING;
	WRESULT ret = WRET_OK;
	ret = SessionManager::getInstance()->session_authorize(wosh_username, new_credentials, this);
	if ( WFAILED(ret) ) {
		return ret;
	 }
	this->state = Session::STATE_OPENED;
	this->username = wosh_username;
	this->usernameLogged = wosh_username;
	if ( new_credentials == NULL )
		this->credentials = NULL;
	else
		this->credentials = new_credentials->clone();
	updatePrompt();
/*
	Log(LOG_VERBOSE, ":login() Connecting 'wosh.Bus.Core'.." );
	BusCore.setMessageHandler(this);
	WRESULT bus_connected = BusCore.connect( _Bus_Core );
	if ( WFAILED(bus_connected) )
		Log(LOG_CRITICAL, ":login() FAILED#%d Connecting Bus (wosh.Bus.Core)..", bus_connected );
*/
	return ret;
 }

WRESULT Session::logout() {
	this->state = Session::STATE_CLOSED;
/*
	WRESULT bus_disconnected = BusCore.disconnect();
	if ( WFAILED(bus_disconnected) )
		Log(LOG_WARNING, ":logout() FAILED#%d DisConnecting Bus (wosh.Bus.Core)..", bus_disconnected );
*/
	return WRET_OK;
 }

WRESULT Session::setState( Session::SESSION_STATE value ) {
	this->state = value;
	SessionManager::getInstance()->session_event( this->state, this );
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT Session::parseInput( const string& user_input ) {
	if ( user_input.empty() ) return WRET_ERR_PARAM;
	setLastInputNow();
	return this->protocol->parseInput( user_input, this );
//			saveHistory(deprecated);
//	return WRET_OK;
 }

WRESULT Session::postMessage( Message* message ) {
	if ( !message->hasSecurityToken() ) {
		if ( this->credentials != NULL )
			this->credentials->signMessage(message, getUsername() );
	 }
	return SessionManager::getInstance()->session_post_message(message, this);
//	message->setSource(this);
//	BusCore.postMessage( message );
	return WRET_OK;;
 }

const std::string& Session::updatePrompt() {
	if ( this->protocol == NULL ) return this->prompt;
	this->prompt = this->protocol->buildPrompt( this );
	return this->prompt;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Session::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;
	if ( !MessageFilter::isMessageTarget(message, getURI()) ) return;
	if ( this->protocol == NULL ) return;
	this->protocol->renderMessage( &message, source, this );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Session::changeToUserHome() {
	URI home;
	home.setKernelLocal();
	home.setPath("Users");
	home.setName(this->username);
	setCurrentLocation(home);
 }

void Session::setLastInputNow() const {
	this->lastInputTS = Utilities::std_time();
 }

WRESULT Session::setWelcomeMessage( const string& value ) {
	if ( this->state == Session::STATE_OPENED ) return WRET_ERR_ILLEGAL_USE;
	this->msgWelcome = value;
	return WRET_OK;
 }

WRESULT Session::setSkipWelcome( bool value ) {
	if ( this->state == Session::STATE_OPENED ) return WRET_ERR_ILLEGAL_USE;
	this->skipWelcome = value;
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

DataFieldCollector<Session>* Session::getDataFields() {
	DataFieldCollector<Session>* fields = new DataFieldCollector<Session>();
	fields->add<long, Variant>(&Session::getID, "ID", 0);
	fields->add<const std::string&, Variant>(&Session::getName, "Name", 1);
	fields->add<const char*, Variant>(&Session::getClassName, "Type", 2);
	fields->add<const char*,Variant>(&Session::getStateAsString, "State", 3);
	fields->add<const std::string&,Variant>(&Session::getUsername, "Username", 4);
	fields->add<long,Variant_DT>(&Session::getLastInputTS, "LastInputTS", 5); ///@bug NOT interpreted as TIMESTAMP!!!!!!
	DataFieldSubWrapperT<Session, const SessionProtocol*, const char*, Variant>* dfwProtocolStr;
	dfwProtocolStr = new DataFieldSubWrapperT<Session, const SessionProtocol*, const char*, Variant>(&Session::getProtocol, &SessionProtocol::getClassName, "ProtocolName", 6);
	dfwProtocolStr->info.visible = false;
	fields->add(dfwProtocolStr);
	fields->add<const std::string&,Variant>(&Session::getEntityID, "EntityID", 7, false);
	fields->add<const URI&,URI>(&Session::getURI, "URI", 8, false);
	fields->add<const Permission&,Permission>(&Session::getPermission, "Permission", 9, false);
	return fields;
 }

const char* SessionStateStr[] = {
	"UNKNOWN",	// (SESSION_STATE)0
	"IDLE",		// (SESSION_STATE)1
	"CONNECTED",// (SESSION_STATE)1
	"LOGGING",	// (SESSION_STATE)1
	"OPENED",	// (SESSION_STATE)1
	"CLOSED"	// (SESSION_STATE)1
 };

Enum2StrFromArray(Session::getStateAsString, Session::SESSION_STATE, STATE_IDLE, STATE_CLOSED, 0, SessionStateStr)

Str2EnumByArray(Session::getStateFromString, Session::SESSION_STATE, 0, STATE_CLOSED, STATE_UNKNOWN, SessionStateStr)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
