/**
 * @class   wosh::Session
 * @brief   
 *
 ****************************************************************************
 * @version $Id: Session.h 2804 2010-07-27 04:39:07Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_Session_H__
 #define __WOSH_Core_Session_H__

 #include <core/Object.h>
 #include <core/Containers.h>
 #include <core/BusConnector.h>
 #include <core/LogContext.h>
 #include <core/SecurityToken.h>

namespace wosh {

#define WOSH_SESSION(CLASSNAME)			\
		WOSH_CLASSNAME(CLASSNAME)		\
		WOSH_CLASS_ABSTRACT(false)

class SessionProtocol;
class SessionManager;
class SessionInfo;
template <class T> class DataFieldCollector;

class Session :	public Object,
				public virtual IBusListener
 {
	WOSH_CLASSNAME(wosh::Session)
	WOSH_CLASS_ABSTRACT(true)

	public:
		/**
		 * @brief   states of a session
		 */
		enum SESSION_STATE {
			STATE_UNKNOWN		= 0x00,
			STATE_IDLE			= 0x01,	///< "allocated"
			STATE_CONNECTED		= 0x02,	///< just connected, going to login, or a failed login (before disconnecting?)
			STATE_LOGGING		= 0x03,	///< logging (user:pass)
			STATE_OPENED		= 0x04,	///< session is active
			STATE_CLOSED		= 0x05	///< session is closed, may be deleted
			// FAILURE
		 };

	public:
	
		Session();
		
		virtual ~Session();

	public:
		virtual WRESULT open( unsigned long timeout = LIMIT_ULONG_MAX ) = 0;
		virtual WRESULT close( unsigned long timeout = LIMIT_ULONG_MAX ) = 0;


/** @name Message Events (incoming message)
 * @{
 ******************************************************************************/
	public:
		// filtered, forwarded by SessionManager
		// (a)synch render and tell to user
		virtual void busMessage( const Message& message, const Bus* source );
//@}

	protected:
		// parsed from user input, rendered to Message
		// see postMessage
		WRESULT parseInput( const std::string& user_input_line );

		/**
		 * @brief  Post a message to Bus, forward message to wosh::SessionManager
		 * @param message [in] the message to be posted, Message is validated
		 * @return refer to wosh::SessionManagersession_post_message()
		 * @note   Implementations must call this method to post messages through SessionManager
		 *         since class friendship is not inherited
		 */
		WRESULT postMessage( Message* message );
		/**
		 * @brief  Send (output) message to user
		 * @param output_text [in] the message to be sent
		 */
		virtual void printMessage( const std::string& output_text ) = 0;


	protected:
		// eval, eventually change state to OPEN
		// sort of security system by source-code point of view too
		
	WRESULT login( const SessionInfo* session_info );
	WRESULT login( const std::string& wosh_username, const SecurityTokenProvider* credentials );
	WRESULT logout();

//@}
/** @name Getters
 * @{
 ******************************************************************************/
 	public:
		inline long getID()	const							{ return this->id; }
		inline const std::string& getUsername() const		{ return this->username; }
		inline const std::string& getUsernameLogged() const	{ return this->usernameLogged; }

		inline long getLastInputTS() const					{ return this->lastInputTS; }

		inline const URI& getCurrentLocation() const 		{ return this->currentLocation; }
		inline const std::string& getCurrentBus() const		{ return this->currentBus; }

		inline bool getSkipWelcome() const					{ return this->skipWelcome; }

		inline const std::string& getPrompt() const			{ return this->prompt; }
		inline const URI& getCreatorURI() const				{ return this->creator; }

		inline const std::string& getWelcomeMessage() const	{ return this->msgWelcome; }

		inline Session::SESSION_STATE getState() const		{ return this->state; }
		inline bool isIdle() const							{ return (this->state == Session::STATE_IDLE); }
		inline bool isConnected() const						{ return (this->state == Session::STATE_CONNECTED); }
		inline bool isOpened() const						{ return (this->state == Session::STATE_OPENED); }
		inline bool isClosed() const						{ return (this->state == Session::STATE_CLOSED); }

		inline const char* getStateAsString() const			{ return getStateAsString(this->state); }

		static const char* getStateAsString( Session::SESSION_STATE state );
		static Session::SESSION_STATE getStateFromString( const char* state );

		static DataFieldCollector<Session>* getDataFields();

//@}
/** name Setters
 * @{
 ******************************************************************************/
 	public:
		virtual void setCurrentLocation( const URI& uri )		{ this->currentLocation = uri; updatePrompt(); }
		virtual void setCurrentBus( const std::string& value )	{ this->currentBus = value; updatePrompt(); }

		const std::string& updatePrompt();

		void changeToUserHome();

		WRESULT setSkipWelcome( bool value );
		WRESULT setWelcomeMessage( const std::string& value );

//		virtual void setUsername( const std::string& value );
//		virtual void setUsernameLogged( const std::string& value );

//@}
/** @name Internal methods, provided for implementations
 * @{
 ******************************************************************************/
	protected:
		WRESULT setState( Session::SESSION_STATE value );

		void setLastInputNow() const;

		inline const SessionProtocol* getProtocol() const	 { return this->protocol; }

	//	virtual void saveHistory( const std::string& value );

	protected:
		friend class wosh::SessionManager;
		friend class wosh::SessionProtocol;

		inline void setProtocol( SessionProtocol* ptr )		{ this->protocol = ptr; }
		inline void setCreatorURI( const URI& uri )			{ this->creator = uri; }
		inline void setUsername( const std::string& user )	{ this->username = user; }

//@}
	protected:
		LogContext Log;

	private:
		long id;				///< session id
		URI creator;			///< uri of my creator
		int64 creation_ts;		///< timestamp of creation
	//	int initDelay;			///< force delay before login, in seconds (may be 0)
	//	long inputTimeout;		///< input timeout (will make session expire), in seconds (may be 0)
		std::string msgWelcome;	///< welcome message (after login)
		bool skipWelcome;		///< skip welcome message
		SESSION_STATE state;	///< session state
		std::string prompt;		///< cached session prompt

		BusConnector BusCore;				///< connector to Core

		std::string username;				///< current user
		std::string usernameLogged;			///< original logged user
		SecurityTokenProvider* credentials;

		SessionProtocol* protocol;			///< the i/o processor (convert user's input and responses)

		mutable URI currentLocation;		///< target URI, it is like a system path (exec 'pwd' to prompt)
		std::string currentBus;				///< target Bus, see Message::destinationBus, default should be wosh.Bus.Core

		mutable long lastInputTS;			///< timestamp of last user input
		mutable ListT<std::string> history; ///< last executed commands/messages, see saveHistory()
		unsigned int historyMaxSize;		///< max items of history, see saveHistory()


}; // class def







class SessionInfo : public IReflection
 {
	WOSH_CLASSNAME(wosh::SessionInfo)
	WOSH_CLASS_ABSTRACT(false)


	public:
		SessionInfo() { 
			this->credentials = NULL;
			this->lastInputTS = 0;
			this->currentLocation.setKernelLocal();
		 }
		virtual ~SessionInfo() { }


	public:
		std::string getCurrentKernel() const {
			MutexLockerRead(&this->mux);
			return this->currentLocation.getKernelName();
		 }
		std::string getUserName() const {
			MutexLockerRead(&this->mux);
			return this->username;
		 }
		std::string getUserNameLogged() const {
			MutexLockerRead(&this->mux);
			return this->usernameLogged;
		 }

		const SecurityTokenProvider* getCredentials() const		{ return this->credentials; }

	public:
		inline const URI& getCurrentLocation() const 			{ return this->currentLocation; }
		inline URI& getCurrentLocation() 						{ return this->currentLocation; }
		inline const std::string& getCurrentBus() const			{ return this->currentBus; }

	public:
		inline void setUsername( const std::string& user )		{ this->username = user; }
		inline void setCredentials( SecurityTokenProvider* stp ) {
			if ( this->credentials != NULL ) delete this->credentials;
			this->credentials = stp;
		 }

	public:	
		inline void transactionBeginRead() const				{ this->mux.lockForRead(); }
		inline void transactionBeginWrite() const				{ this->mux.lockForWrite(); }
		inline void transactionEnd() const						{ this->mux.unLock(); }

		inline wosh::MutexRW* getMutex() const					{ return &this->mux; }


	public:
		SecurityToken* createToken( const Object* creator = NULL) {
			MutexLockerRead(&this->mux);
			if ( this->credentials == NULL ) return NULL;
			return this->credentials->createToken( creator, this->username );
		 }

		SecurityToken* signMessage( Message* message ) {
			MutexLockerRead(&this->mux);
			if ( this->credentials == NULL ) return NULL;
			return this->credentials->signMessage( message, this->username );
		 }

		SecurityToken* signMessage( const Object* creator, Message* message ) {
			MutexLockerRead(&this->mux);
			if ( this->credentials == NULL ) return NULL;
			return this->credentials->signMessage( message, creator );
		 }

	protected:
		std::string username;				///< current user
		std::string usernameLogged;			///< original logged user
		SecurityTokenProvider* credentials;

		mutable URI currentLocation;		///< target URI, it is like a system path (exec 'pwd' to prompt)
		std::string currentBus;				///< target Bus, see Message::destinationBus, default should be wosh.Bus.Core

		mutable long lastInputTS;			///< timestamp of last user input

		mutable MutexRW mux;


}; // class def




}; // namespace wosh

#endif //__WOSH_Core_Session_H__
