/** @file    SharedMemoryEpidemic.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SharedMemoryEpidemic.cpp 2664 2010-06-16 16:53:08Z alex $
 *  @brief
 * File containing methods for the wosh::SharedMemoryEpidemic class.
 * The header for this class can be found in SharedMemoryEpidemic.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <core/SharedMemoryEpidemic.h>

 #include <core/ObjectAllocator.h>
 #include <core/SecurityManager.h>
 #include <core/MessageFilter.h>
 #include <core/Request.h>
 #include <core/Response.h>


using namespace std;
namespace wosh {

 WOSH_REGISTER(wosh::SharedMemoryEpidemic, "wosh::SharedMemory", 0.05, _static_SharedMemoryEpidemic)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SharedMemoryEpidemic::SharedMemoryEpidemic( const std::string& name, Object* parent )
	: SharedMemory(name, parent) {
	this->info.strategy = static_cast<short>(SharedMemory::STRATEGY_EPIDEMIC);
	this->info.propagation = static_cast<short>(SharedMemory::PROPAGATION_REPLICA);
	this->spread_mode = SharedMemoryEpidemic::SPREAD_ANTIENTROPY;
 }

SharedMemoryEpidemic::~SharedMemoryEpidemic() {
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// FLOW/CONTROL

WRESULT SharedMemoryEpidemic::init( unsigned long timeout ) {
	WRESULT ret = SharedMemory::init(timeout);
	if ( WFAILED(ret) ) {
		Log(LOG_CRITICAL, ":init() FAILED#%i : base initializer!", ret );
		return ret;
	 }
	//Log(LOG_DEBUG, ":init() : Starting intenal thread.." );
	//ret = startThread(timeout);
	return ret;
 }

WRESULT SharedMemoryEpidemic::deinit( unsigned long timeout ) {
	//Log(LOG_VERBOSE, ":deinit() : Stopping intenal thread.." );
	WRESULT ret = WRET_OK;
	//ret = quitThread(timeout);
	//if ( WFAILED(ret) ) {
	//	Log(LOG_CRITICAL, ":deinit() FAILED#%i : Stopping thread!", ret );
	//	return ret;
	// }
	ret = SharedMemory::deinit(timeout);
	return ret;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// FLOW/CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////// SETUP

WRESULT SharedMemoryEpidemic::setMode( SharedMemory::AGENT_MODE mode, bool raise_update ) {
	if ( mode == SharedMemory::MODE_UNKNOWN ) return WRET_ERR_PARAM;
	Log(LOG_INFO, ":setMode() : Setting to %i", (int)mode );
	this->agents.transactionBeginWrite();
	this->info.mode = static_cast<short>(mode);
	this->agents.transactionEnd();
	if ( raise_update )
		doElectionRequest_(0);
	return WRET_OK;
 }

WRESULT SharedMemoryEpidemic::setPropagation( SharedMemory::UPDATE_PROPAGATION propagation, bool raise_update ) {
	if ( propagation == SharedMemory::PROPAGATION_UNKNOWN ) return WRET_ERR_PARAM;
	Log(LOG_INFO, ":setPropagation() : Setting to %i", (int)propagation );
	this->agents.transactionBeginWrite();
	this->info.propagation = static_cast<short>(propagation);
	this->agents.transactionEnd();
	if ( raise_update )
		doElectionRequest_(0);
	return WRET_OK;
 }

WRESULT SharedMemoryEpidemic::setSpreadMode( SharedMemoryEpidemic::SPREAD_MODE spread, bool raise_update ) {
	if ( spread == SharedMemoryEpidemic::SPREAD_UNKNOWN ) return WRET_ERR_PARAM;
	Log(LOG_INFO, ":setSpreadMode() : Setting to %i", (int)spread );
	this->agents.transactionBeginWrite();
	this->spread_mode = spread;
	this->agents.transactionEnd();
	if ( raise_update )
		doElectionRequest_(0);
	return WRET_OK;
 }

SharedMemory::AGENT_MODE SharedMemoryEpidemic::getMode() const {
	MutexLockerRead(this->agents.getMutex());
	return static_cast<SharedMemory::AGENT_MODE>(this->info.mode);
 }
 
SharedMemory::UPDATE_PROPAGATION SharedMemoryEpidemic::getPropagation() const {
	MutexLockerRead(this->agents.getMutex());
	return static_cast<SharedMemory::UPDATE_PROPAGATION>(this->info.propagation);
 }

SharedMemoryEpidemic::SPREAD_MODE SharedMemoryEpidemic::getSpreadMode() const {
	MutexLockerRead(this->agents.getMutex());
	return this->spread_mode;
 }

/////////////////////////////////////////////////////////////////////////////////////////////////////// SETUP
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////// DATASET

WRESULT SharedMemoryEpidemic::doObjectAdd( SharedObject* object ) {
	if ( object == NULL ) return WRET_ERR_PARAM;
	this->dataset.transactionBeginWrite();
	tStrSharedObjectMap::Iterator it;
	SharedObject* sobj = this->dataset.find( object->getSharedID() );
	if ( sobj == object ) {
		Log(LOG_CRITICAL, ":doObjectAdd(%s, [%s]) FAILED : Object exists (same id,pointer)", object->getSharedID().c_str(), object->getClassName() );
		this->dataset.transactionEnd();
		return WRET_ERR_PARAM;
	 }
	else if ( sobj != NULL ) {
		Log(LOG_CRITICAL, ":doObjectAdd(%s, [%s]) FAILED : Object exists (same id, different pointer)!", object->getSharedID().c_str(), object->getClassName() );
		this->dataset.transactionEnd();
		return WRET_ERR_PARAM;
	 }
	else {
		Log(LOG_INFO, ":doObjectAdd(%s) Object Added! [kind-of %s]", object->getSharedID().c_str(), object->getClassName() );
		this->dataset.set(object->getSharedID(), object);
		spreadUpdate( object, SharedObject::OBJ_CREATED );
	 }
	this->dataset.transactionEnd();
	return WRET_OK;
 }

WRESULT SharedMemoryEpidemic::doObjectUpdated( SharedObject* object ) {
	if ( object == NULL ) return WRET_ERR_PARAM;
	this->dataset.transactionBeginWrite();
	tStrSharedObjectMap::Iterator it;
	SharedObject* sobj = this->dataset.find( object->getSharedID() );
	if ( sobj == NULL ) {
		Log(LOG_CRITICAL, ":doObjectUpdated(%s, [%s]) FAILED : Object DOES NOT exist!", object->getSharedID().c_str(), object->getClassName() );
		this->dataset.transactionEnd();
		return WRET_ERR_PARAM;
	 }
	else if ( sobj != object ) {
		Log(LOG_CRITICAL, ":doObjectUpdated(%s, [%s]) FAILED : Object exists BUT (same id, different pointer)!", object->getSharedID().c_str(), object->getClassName() );
		this->dataset.transactionEnd();
		return WRET_ERR_PARAM;
	 }
	else {
		Log(LOG_INFO, ":doObjectAdd(%s) Object Updated! [kind-of %s]", object->getSharedID().c_str(), object->getClassName() );
		spreadUpdate( object, SharedObject::OBJ_CREATED );
	 }
	this->dataset.transactionEnd();
	return WRET_OK;
 }

//////////////////////////////////////////////////////////////////////////////////////////////// FLOW/CONTROL
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SharedMemoryEpidemic::spreadUpdate( SharedObject* object, SharedObject::OPERATION action ) {
	if ( object == NULL ) return WRET_ERR_PARAM;
	return spreadUpdate(object->getSharedID(), object, action, 0);
 }

WRESULT SharedMemoryEpidemic::spreadUpdate( const std::string& object_id, SharedObject* object, SharedObject::OPERATION action, long skip_target ) {
	if ( !isModeServer() ) {
		Log(LOG_CRITICAL, ":spreadUpdate() FAILED : Not allowed in CLIENT mode" );
		return WRET_ERR_ILLEGAL_USE;
	 }
	// select target agents
	this->agents.transactionBeginRead();
	std::vector<long> selected_targets;
	selectUpdateTargets_(selected_targets, skip_target);
	this->agents.transactionEnd();
	if ( selected_targets.size() == 0 ) {
		Log(LOG_CRITICAL, ":spreadUpdate() FAILED : No target Agents selected!" );
		return WRET_ERR_ILLEGAL_USE;
	 }
	// select how to spread the update
	MessageContent* content = NULL;
	switch( getPropagation() ) {
		case PROPAGATION_INVALIDATE: {
				content = new SharedMemory_ObjectInvalidation(object_id, action);
				break;
			 }
		case PROPAGATION_REPLICA: {
				if ( object == NULL ) {
				 }
				else {
					content = new Response(_SharedMemory_METHOD_retrieve, object->clone());
				 }
				break;
			 }
		case PROPAGATION_ACTIVE:
		case PROPAGATION_UNKNOWN:
		default: {
				Log(LOG_CRITICAL, ":spreadUpdate() FAILED : Unknown/invalid PROPAGATION mode [%i]!", (int)getPropagation() );
				return WRET_ERR_ILLEGAL_USE;
			 }
	 }
	// create and send the request
	Message message;
	message.setContent(content);
	std::vector<long>::const_iterator it_s;
	for ( it_s=selected_targets.begin(); it_s!=selected_targets.end(); ++it_s) {
		sendToAgent(message.clone(), *it_s);
	 }
	return WRET_OK;
 }

void SharedMemoryEpidemic::selectUpdateTargets_( std::vector<long>& selected_targets, long skip_target ) {
	switch( this->spread_mode ) {
		case SPREAD_ANTIENTROPY: {
				// pure anti-entropy algorithm:  agent chooses another at random
				selected_targets.push_back( selectRandomAgent_() );
				break;
			 }
		case SPREAD_GOSSIPING: {
				///@bug TO BE IMPLEMENTED
				// Gossiping is a specific variant of anti-entropy
				// - If P has just been updated for data item x, it contacts some arbitrary Q and tries to push x to Q
				// - If Q was already update, P has a chance of losing interest in spreading the update further - if it doesnt, it picks another Q and tries again
				// The combination of gossiping and occasional pulls can ensure that all servers are updated quickly
				selected_targets.push_back( selectRandomAgent_() );
				break;
			 }
		case SPREAD_BROADCAST: {
				selected_targets.push_back( 0 ); // 0 is a special value for sendToAgent()
				break;
			 }
		case SPREAD_UNKNOWN:
		default: {
				Log(LOG_CRITICAL, ":selectUpdateTargets_() FAILED : Unknown/invalid PROPAGATION mode [%i]!", (int)getPropagation() );
			 }
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SharedMemoryEpidemic::gotInvalidationOf( const std::string& object_id, SharedObject::OPERATION action, long sender_id, long target_id ) {
	// parse and eventually apply the update, forward the infection
	this->dataset.transactionBeginWrite();
	SharedObject* sobj = this->dataset.find(object_id);
	if ( action == SharedObject::OBJ_DELETED ) {
		// really delete the shared object BUT keep the (NULL) entry
		if ( sobj == NULL ) {
			Log(LOG_WARNING, ":gotInvalidationOf(%s) : OBJ_DELETED from %ld (not forwarding infection)", object_id.c_str(), action, sender_id );
			// probably we got this update already! don't forward
		 }
		else {
			Log(LOG_INFO, ":gotInvalidationOf(%s) : OBJ_DELETED from %ld", object_id.c_str(), action, sender_id );
			this->dataset.erase(object_id);
			this->dataset.set(object_id, NULL);
		 }
		if ( target_id != 0 ) { // not broadcast
			spreadUpdate( object_id, NULL, action, sender_id );
		 }
		// apply SPREAD_GOSSIPING algorithm
	 }
	else {
		// let's request(pull) the object
		sendToAgent( new Message(new Request(_SharedMemory_METHOD_retrieve, new Variant(object_id))), sender_id );
	 }

	this->dataset.transactionEnd();
 }

void SharedMemoryEpidemic::gotObject( const SharedObject* object, long sender_id, long target_id ) {
	if ( object == NULL ) return;
	// got a pull response, or a push
	this->dataset.transactionBeginWrite();
	SharedObject* sobj = this->dataset.find( object->getSharedID() );
	if ( sobj == NULL ) {
		this->dataset.set(object->getSharedID(), object->clone()->as<SharedObject>() );
	 }
	else {
		*sobj = *object;
	 }
	this->dataset.transactionEnd();
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS

void SharedMemoryEpidemic::busMessage( const Message& message, const Bus* source ) {
	if ( message.isEmpty() ) return;
	SharedMemory::busMessage( message, source );
	if ( !MessageFilter::isMessageTarget(message, Object::getURI() ) ) return;
	long sender_id = 0;	long target_id = 0;
	if ( !message.getSource().isNameAny() ) sender_id = Utilities::fromString<long>(message.getSource().getName());
	if ( !message.getDestination().isNameAny() ) target_id = Utilities::fromString<long>(message.getDestination().getName());

	if ( message.getContent()->isFact() ) {
		if ( message.getContent()->isKindOf<SharedMemory_ObjectInvalidation>() ) {
			const SharedMemory_ObjectInvalidation* oinv = message.getContent()->as<SharedMemory_ObjectInvalidation>();
			if ( oinv == NULL ) return;
			gotInvalidationOf( oinv->getObjectID(), oinv->getUpdateOperation(), sender_id, target_id );
		 }
	 }
	else if ( message.getContent()->isRequest() ) {
		const Request* request = message.getContent()->asRequest();
		if ( request == NULL ) return;
		if ( request->getMethod() == _SharedMemory_METHOD_retrieve ) {
			// got a pull request, so push the selected object
			Message* reply = NULL;
			Variant obj_id;
			request->extractArguments(obj_id);
			this->dataset.transactionBeginRead();
			SharedObject* sobj = this->dataset.find( obj_id.toString() );
			if ( sobj == NULL )
				reply = message.replyTo( request->replyResponse(WRET_ERR_PARAM, obj_id.clone()) );
			else
				reply = message.replyTo( request->replyResponse(WRET_OK, sobj->clone()) );
			this->dataset.transactionEnd();
			reply->setSource(this);
			SecurityManager::getCredentialImplicit().signMessage(reply, this);
			BusDSM.postMessage(reply);
		 }
	 }
	else if ( message.getContent()->isResponse() ) {
		const Response* response = message.getContent()->asResponse();
		if ( response == NULL ) return;
		if ( response->getMethod() == _SharedMemory_METHOD_retrieve ) {
			if ( !response->hasData() ) return;
			if ( WFAILED(response->getReturnValue()) ) return;
			const SharedObject* sobj_new = response->getData()->as<SharedObject>();
			gotObject(sobj_new, sender_id, target_id );
		 }
	 }
 }

////////////////////////////////////////////////////////////////////////////////////////////////// BUS EVENTS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

}; // namespace wosh
