/**
 * @class   wosh::SharedMemoryEpidemic
 * @brief   SharedMemory implementation based on Epidemic algorithm.
 *
 * An epidemic protocols goal is to propagate updates to all replicas using as few messages as possible. 
 *
 * They do not do any conflict resolution among updates, because of that
 * it should be used for data stores that only need eventual consistency.
 *
 * Theorical properties:
 *  - Intrinsically distributed and redundant
 *  - Scalable (fanout proportional to log(nodes))
 *  - Fault-tolerant
 *  - Resilient to topological changes
 *  - Probabilistic in nature
 *
 * Intrinsic/Implementation properties:
 *  - PROPAGATION_ACTIVE (active propagation) is not available
 *  - MODE_CLIENT never spread updates
 *
 *
 * Flows:
 *
 * References:
 * \see     http://deneb.cs.kent.edu/~mikhail/classes/es.u02/Research/epidemic.ppt
 ****************************************************************************
 * @version 0.8.499 $Id: SharedMemoryEpidemic.h 2665 2010-06-17 20:10:44Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Core_SharedMemoryEpidemic_H__
 #define __WOSH_Core_SharedMemoryEpidemic_H__

 #include <woshDefs.h>
 #include <core/SharedMemory.h>


namespace wosh {


class SharedMemoryEpidemic : public SharedMemory
 {
	WOSH_CLASSNAME(wosh::SharedMemoryEpidemic)
	WOSH_CLASS_ABSTRACT(false)

	public:
		// define how the target are selected when spreading the updates

		/**
		 * @brief define how the target are selected when spreading the updates
		 * @see   selectUpdateTargets_()
		 */
		enum SPREAD_MODE {
			SPREAD_UNKNOWN			= 0x00,	///< undefined, for initialization
			SPREAD_ANTIENTROPY		= 0x01,	///< random agent selection (see selectRandomAgent_)
			SPREAD_GOSSIPING		= 0x02,	///< NOT IMPLEMENTED YET
			SPREAD_BROADCAST		= 0x03	///< update is broadcasted (destination=any kernel/sm)
		 };

		enum INFECTION_STATE {
			STATE_UNKNOWN			= 0x00,	///< undefined, for initialization
			STATE_INFECTIVE			= 0x01,	///< there is an update that it is willing to spread
			STATE_SUSCEPTIBLE		= 0x02,	///< an agent that has not yet been updated
			STATE_REMOVED			= 0x03	///< unwilling or unable to spread the update further
		 };

	public:
		/**
		 * @brief  Default constructor. May initialize Shared-Memory's name and parent reference.
		 * @param name [in] name of the tuplespace, it refers to an unique dataset. An empty will be initialized with a random id.
		 * @param parent [in] parent's reference, may be NULL.
		 */
		SharedMemoryEpidemic( const std::string& name = "", Object* parent = NULL );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~SharedMemoryEpidemic();

	public:
		virtual WRESULT doObjectAdd( SharedObject* object );
		virtual WRESULT doObjectUpdated( SharedObject* object );


/** @name Flow control
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT init( unsigned long timeout = LIMIT_ULONG_MAX );
		virtual WRESULT deinit( unsigned long timeout = LIMIT_ULONG_MAX );

//@}
/** @name Setters (reentrant)
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Set 
		 * @param mode [in] the event string to be assigned
		 * @return WRET_ERR_PARAM if mode is invalid; WRET_OK else.
		 * @see    setContentContext()
		 */
		virtual WRESULT setMode( SharedMemory::AGENT_MODE mode, bool raise_update = true );
		/**
		 * @brief  Set 
		 * @param propagation [in] the event string to be assigned
		 * @return WRET_ERR_PARAM if propagation is invalid; WRET_OK else.
		 * @see    spreadUpdate()
		 */
		virtual WRESULT setPropagation( SharedMemory::UPDATE_PROPAGATION propagation, bool raise_update = true );
		/**
		 * @brief  Set 
		 * @param spread [in] the event string to be assigned
		 * @return WRET_ERR_PARAM if spread is invalid; WRET_OK else.
		 * @see    selectUpdateTargets_()
		 */
		virtual WRESULT setSpreadMode( SharedMemoryEpidemic::SPREAD_MODE spread, bool raise_update = false );

//@}
/** @name Getters (reentrant)
 * @{
 ******************************************************************************/
	public:
		/**
		 * @brief  Get Mode
		 * @return MessageContent::AGENT_MODE
		 */
		AGENT_MODE getMode() const;
		/**
		 * @brief  Get Propagation
		 * @return MessageContent::UPDATE_PROPAGATION
		 */
		UPDATE_PROPAGATION getPropagation() const;
		/**
		 * @brief  Get SpreadMode
		 * @return MessageContent::SPREAD_MODE
		 */
		SPREAD_MODE getSpreadMode() const;

//@}
/** @name IBusListener interface
 * @{
 ******************************************************************************/
	public:
		virtual void busMessage( const Message& message, const Bus* source );
//@}
	protected:
		virtual WRESULT spreadUpdate( SharedObject* object, SharedObject::OPERATION action = SharedObject::OBJ_UPDATED );
		virtual WRESULT spreadUpdate( const std::string& object_id, SharedObject* object, SharedObject::OPERATION action, long skip_target = 0 );

		virtual void selectUpdateTargets_( std::vector<long>& selected_targets, long skip_target = 0 );

		virtual void gotInvalidationOf( const std::string& object_id, SharedObject::OPERATION action, long sender_id, long target_id );
		virtual void gotObject( const SharedObject* object, long sender_id, long target_id );


	protected:
		//bool initializingThread();
		//void runThread();
		//void exitingThread();

	protected:
		SPREAD_MODE spread_mode;

}; // class def




}; // namespace wosh

#endif //__WOSH_Core_SharedMemoryEpidemic_H__
