/**
 * @file    framework/ai.h
 * @brief   Include all components of the ai framework
 ****************************************************************************
 * @version 0.00 $Id: ai.h 2656 2010-06-15 15:04:28Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! @namespace wosh::ai
 * @brief   Classes, algoritms and utilities for applied Artificial Intelligence and Data Mining.
 * @see     \ref page_ai
 * @see     \ref page_framework_ai
 *
 ****************************************************************************/
/*! \page page_framework
 * \htmlonly <hr/> \endhtmlonly
 * \section page_framework_ai WOSH Artificial-Intelligence Framework
 *
 * \warning WOSH AI framework is still under planning.
 *
 * @see \ref page_ai
 *
 ****************************************************************************/
/*! \page page_ai Artificial Intelligence
 *
 * \warning
 *  Artificial intelligence is NOT implemented yet.
 *  Actually is under planning.
 *
 *
 * \section page_ai_toc Table of Contents:
 *
 *  - \ref page_ai_intro
 *
 *  - \ref page_ai_overview
 *
 *
 * \par
 *  "To err is human, but to really foul things up requires a computer"
 *  (Farmers' Almanac, 1978)
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_ai_intro Introduction
 *
 * This is an introduction to applied AI in WOSH system. Topics are exposed and
 * analyzed by this point of view. There are plenty of tutorials on the Internet
 * about AI principles and abstract concepts, but very few applied AI samples.
 *
 *
 * Smart agents are based on PAGE principle: percept, actions, goals, environment. An implementation may look like:
 * \code
 *  function abstractAgent( perception ) {
 *    static memory_knowledge;
 *    memory_knowledge = updateMemory(memory_knowledge, perception);
 *    action = chooseBestAction(memory_knowledge);
 *    memory_knowledge = updateMemory(memory_knowledge, action);
 *    return action;
 *   }
 * \endcode
 * The simplest flow follows if-then rule: on \c event then \c act
 *
 *
 * The agents in a multi-agent system have several important characteristics:
 *  - Autonomy: the agents are at least partially autonomous
 *  - Local views: no agent has a full global view of the system, or the system is too complex for an agent to make practical use of such knowledge
 *  - Decentralization: there is no designated controlling agent (or the system is effectively reduced to a monolithic system)
 *
 * WOSH architecture is compatible with this point of view: an agent is a bundle.
 * Usually there will be a set of bundles, one smart and some dummies, acting as a smart agent.
 *
 * The key idea is to enable services intelligence providing a base set of classes and functionalities.
 *
 * A more specific scenario, very important within smart home, is HMI (Humain Maichen Interface).
 * This topic may be disassembled in three key points: undestand request, perform actions, report results.
 *
 * The WOSH Shell system is a dummy sample: well-known command/syntax is parsed (from console/IM-session),
 * converted into a message wosh::Request (and sent), then wosh::Response is parsed and converted into text
 * to be presented to the final user.
 *
 * A smart interface, human-friendly or even natural-speech-enabled must still be able to convert the input
 * to a standard system request.
 *
 * Let's consider the input was already converted to a plain text.
 * Understanding of Human language is implemented using one of the following engines:
 *   - Keyword Based : simple algorithms based on keyword database and comparison. Mostly used within Bots (example: IRC).
 *   - Rule Based : usually based on forward chaining, they are simplest Expert system.
 *   - Expert Systems : they are built for specific context, the set of rules is the main point while implementing the application.
 *   - Neural Networks : they IT version of Human brain.
 *
 * In most implementation there is a system requirement: Data Mining.
 * Let's introduce the topic with an abstract flow:
 * \verbatim
HMI-INPUT: turn off music 
HMI-PARSER=> analyze input: action{"turn off"}; TARGET{"music"}
HMI-MAPPER=> best-map: TARGET music=> :/MediaDirector
HMI-MAPPER=> best-map: ACTION turn off=> stop
HMI-FORWARDER=> send Message(new Request("stop")) to ":/MediaDirector"
[..] asynch
HMI-FEEDBACK=> analyze Response(WRET_OK) from ":/MediaDirector"
HMI-FEEDBACK=> map to previous request session.
HMI-REPLY=> convert to human language {"turn off music"=>WRET_OK}
\endverbatim 
 *
 * This flow is enabled by two core components:
 *  - Human parser/generator
 *  - a smart mapping component
 *
 * It's quite simple to understand that both components require a knowledge of the
 * system and its classification. The mapper would also need real-time search.
 * Here comes Data Mining: the association of information to object.
 *
 * Data Mining... [..] {}
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_ai_overview Overview: AI within WOSH
 *
 * \warning AI framework is still under planning.
 *
 *
 * Artificial Intelligence applied into WOSH framework may be related and improve
 * many different aspects and functionalities of the system, most important are:
 *  - Data mining and classification (which is required by next point)
 *  - Human interaction (such as a "lame" natural language parser/producer)
 *  - Problem solving: of unknown (not deterministic) states
 *  - Search: finding services, objects, methods from an human perspective
 *  - Prediction: of users' action, system states
 *  - recognize/understand/predict generic stuff.. (for example about inhabitants' lifestyle and interaction)
 *  - so.. classes/"sub-framework" that allows other components to include AI easily and transparently
 *
 * \par real world example:
 *  Talking about integration, here are some use-cases which could help you understand goals and requirements:
 *    - Communication: upgrade the communication system/architecture in order to pick best channel and preferences for an user,
 *      since there are already many ways they could interact.
 *      Actually I wrote a 'lame' dynamic ranking system to select the best communicator (because of I was mostly interested
 *      in the whole communication-architecture and having something which already works).
 *      An upgrade could use GA and (dynamic) fitness functions to select such parameters.
 *
 *    - User interaction: I usually control the system using a sort of (POSIX) shell syntax,
 *      which is good when communicator is a stdin/stdout console or a messaging client as GTalk,
 *      but it's unacceptable when messages are SMS or when they should be converted in human language (Text2Speech).
 *
 *  On GTalk, to play first element of the playlist, I could write: 
 * \verbatim
cd Bundles
cd PlayerHouse
first
\endverbatim
 * which would reply "Method: first returned OK [No data]"
 *
 * Writing such commands in an SMS message (and having such reply back) is far away to be user-friendly.
 * It should look like "play music in whole house" and its reply: "playlist started, now playing xx".
 * Obtaining that.. means having a custom (smart) SessionProtocol implementation which is able to interpret a simple language
 * (even just based on keywords at beginning) and bind it to a component/method request (as the response).
 * Basically the system must have a (dynamic, almost realtime) knowledge of the whole system and its component,
 * in other words: data-mining and classification of components and its features.
 * 
 *
 *
 * I won't include a custom AI engine into WOSH, the key idea is to integrate
 * third-party libraries and implement just some common adapters and interfaces.
 *
 * The external libraries, indeed, must be licensed as open source, stable
 * and it would be nice if they are still alive.. 
 * 
 * The first chosen library is: \b galib "GAlib: Matthew's Genetic Algorithms Library"
 * see \ref page_ai_genetic further in this page.
 *
 * Check out \ref page_build_3rdparty document for information about WOSH
 * generic approach (of building).
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_ai_genetic Genetic Algorithms
 *
 * \b "GAlib: Matthew's Genetic Algorithms Library" has been developed at MIT
 * some years ago (last update is in 2005).
 *
 * GAlib is open source, multiplatform, well designed, extensible. It compiles
 * naturally on POSIX and updating some project's settings, also with MSVC9.
 *
 * GAlib is available at <a href="http://lancet.mit.edu/ga/">http://lancet.mit.edu/ga/</a>.
 *
 *
 *
 *
 * \htmlonly <hr/> \endhtmlonly
 ****************************************************************************
 * \section page_ai_web Web References
 *
 *  - http://www.pcai.com/
 *  - Data Mining (http://www.docstoc.com/docs/16264887/Use-of-Object-Oriented-Concepts-in-Databases-for-Effective-Mining)
 *
 *
 ****************************************************************************
 *








http://people.cs.uu.nl/westra/articles/scriptie.pdf
Fuzzy logic is a way to express a non-linear function of something. This can
be used for things where we do not want to know if something is true or not,
but when you want to know how good (or even how likely) something is.








 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_FrameWork_AI_H__
 #define __WOSH_FrameWork_AI_H__

 #include <woshDefs.h>

namespace wosh {
 namespace ai { };
};

 #include <framework/ai/genetic.h>
 #include <framework/ai/greedy.h>



#endif //__WOSH_FrameWork_AI_H__







/*
Autonomic systems typically comprise three key elements:
monitoring, decision-making, and reconguration. Moni-
toring enables an application to be aware of its environ-
ment and detect conditions that warrant reconguration;
decision-making determines which set of monitored condi-
tions should trigger a specic reconguration response; and
reconguration enables an application to change itself in
order to fulll its requirements. Many adaptive and au-
tonomic systems have applied rule-based decision-making
approaches to match particular events against specic re-
conguration plans


==========================================================================================
Traditional AI expert systems records (endcodes) human knowledge in the form of heurictic rules.
Learning methods might be used to automate decision-macking, adapt to design changes, but training an AI can be very difficult and time consuming.
Finite state machine (FSM) architectures are frequenctly used as basis for AI, but then can be extended using OOP (state pattern).
Each AI engine owns a container of AI_State, a state may evaluate (and declare) its utility:
float AI_State::getUtility( _Context_ ) const;
Expected value of an action is its average payoff.
E(v) = SUM( Pi * Ri )
Sum of all possible outcomes i: of the probability i multiplied by reward i

Game Programming Gems 4
==========================================================================================

AI engines are generally centralized, ..



==========================================================================================
Ideally, autonomous agents should continuously monitor their world and react accordingly.
Every frame they might poll the world for events or objects to which they should
respond. This means everything from collision detection to noticing interesting things
to look at. The problem with diis approach is that it promotes an enormous amount of
redundant computation. As a rule, individual polling is usually wasteful.
The alternative is to employ event-driven techniques whenever possible. For
example, in a baseball game when the ball is initially hit, the ball could simply tell the
fielders that it was hit. This is extremely more efficient than each fielder polling the
situation to detect when the batter has hit the ball. Granted, this is an obvious use of
event-driven behavior, but it still shows the magnitude of processing that can be saved
by using an event-driven strategy.
																=> perfect with WOSH
==========================================================================================


When given a large problem, sometimes it suffices to only solve the part that you need
right away. The rest of the solution can then be computed in the future when you
actually need it. It might even be the case that the situation changes enough that the
rest of the problem is irrelevant and never needs to be computed anyway. This strategy
is extremely similar to lazy evaluation.
The best example of this strategy in action is probably hierarchical pathfinding. In
hierarchical pathfinding, the path is found in two phases. First, the character's highlevel,
room-to-room path to the goal is calculated. Once that is computed, the micro
path to get the character to the next room, toward the goal, can be computed as each
new room is entered. If for some reason the character gets redirected, the remaining
path is thrown away and never computed. This on-demand pathfinding results in a
tremendous speed improvement that is critical for games that have large areas to navigate.



==========================================================================================

Influence mapping is an invaluable and proven game AI technique for performing
tactical assessment. Influence maps have been used most often in strategy games, but
are also useful for many other types of games that require an aspect of tactical analysis.
The general concepts of influence mapping are an essential part of any AI developer's
toolkit.
An influence map is a spatial representation of an AI agent's knowledge about the
world. It allows a computer player to develop a tactical perspective of the current
game state layered on top of the underlying physical/geographical representation of
the game environment.
												=> quanto interagire in un determinato campo
==========================================================================================


"Fuzzy logic is a superset of conventional (Boolean) logic that has been
extended to handle the concept of partial truthtruth values between
"completely true" and "completely false" [FAQ97]."



*/




