/**
 * @file    framework/automations.h
 * @brief   Include all components of the automations framework
 ****************************************************************************
 * @version 0.8.013 $Id: automations.h 2258 2010-04-02 13:02:21Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! @namespace wosh::automations
 * @brief   Automation and Task (Cron) related classes.
 *
 * The interface of and WOSH automation is wosh::automations::Automation (abstract) class.
 *
 * Some very flexible implementations are provided:
 *  - wosh::automations::AutomationMessage - triggered by filtered messages, generate custom messages
 *  - wosh::automations::CronJob - triggered at a custom time or periodically, generate custom messages
 *
 *
 * @see Refer also to wosh::services::TaskManagerBundle.
 * @see \ref page_framework_automations
 *
 ****************************************************************************/
/*! \page page_framework
 * \htmlonly <hr/> \endhtmlonly
 * \section page_framework_automations WOSH Automation Framework 
 *
 * Automations are generic actions executed by a generic trigger.
 * Since WOSH is a message-oriented system, action and triggers are both messages.
 *
 * wosh::Automation is the base class of a generic automation, automations are
 * loaded and managed by wosh::services::AutomationManagerBundle.
 *
 * In short, Automation Manager will monitor busses and check if each wosh::Message
 * is a valid trigger of any automation, eventually executing it.
 *
 *
 * WOSH comes with a sample archive (created for tests) of automations
 * stored in <a href="../../var/database/automations/automations.xml">/var/database/automations/automations.xml</a>.
 *
 * A set of common implementations ships with WOSH Framework:
 *  - wosh::automations::AutomationMessage -
 *    triggered by one or more messages (as wosh::MessageFilter)
 *    and generating one or more effects (as wosh::Message).
 *
 *  - wosh::automations::AutomationDeviceLink
 *    Automation mapping devices events with method calls, supports multiple mapping,
 *    triggers are wosh::Fact and generated messages are posted on wosh.Bus.Devices.
 *
 *  - wosh::automations::AutomationFollowing - out of date, deprecated
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_FrameWork_Automations_H__
 #define __WOSH_FrameWork_Automations_H__

 #include <woshDefs.h>

namespace wosh {
 namespace automations { };
};

 #include <framework/automations/Automation.h>
 #include <framework/automations/AutomationMessage.h>
 #include <framework/automations/AutomationDeviceLink.h>
 #include <framework/automations/CronJob.h>


#endif //__WOSH_FrameWork_Automations_H__
