/**
 * @class   wosh::building::Door
 * @brief   BuildingObject representing a generic (real) door.
 *          This class provides standard Methods and properties to 
 *          control the mapped wosh::interfaces::devices::Switch
 *          wosh::building::Door is usually a child of wosh::building::Room
 *
 ****************************************************************************
 * @version 0.8.399 $Id: Door.h 2732 2010-07-06 03:18:11Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_Door_H__
 #define __WOSH_Building_Door_H__

 #include <framework/building/BuildingObject.h>
 #include <interfaces/devices/Switch.h>


namespace wosh {
 namespace building {

 #define _Door_VERSION					0.8399

 #define _Door_KEY_ExternalDoor			"ExternalDoor"
 #define _Door_KEY_AccessTo				"AccessTo"

 #define _Door_METHOD_open				"open"
 #define _Door_METHOD_close				"close"
 #define _Door_METHOD_lock				"lock"
 #define _Door_METHOD_unlock			"unlock"

 #define _Door_EVENT_DoorStateChanged	"DoorStateChanged"
 #define _Door_EVENT_DoorLockedChanged	"DoorLockedChanged"


class Door : public BuildingObject
 {
	WOSH_BUILDINGOBJECT(wosh::building::Door)

	public:
		/**
		 * @brief   door states
		 */
		enum DOOR_STATE {
			DOOR_UNKNOWN	= 0x00, ///< 
			DOOR_OPENED		= 0x01, ///< 
			DOOR_CLOSED		= 0x02, ///< 
			DOOR_LOCKED		= 0x04 ///< 
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Door();
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Door( const Door& m ) : BuildingObject(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Door() { }


	public:
		virtual void busMessageChild( const Message& message );

//@}
/** @name Door Interface - 
 * @{
 ******************************************************************************/
	public:
		virtual WRESULT open()							{ return mmDoOpen(NULL); }
		virtual WRESULT close()							{ return mmDoClose(NULL); }
		virtual WRESULT lock()							{ return mmDoLock(NULL); }
		virtual WRESULT unlock()						{ return mmDoUnlock(NULL); }

	public:
		virtual std::string getMainSwitchName() const	{ return getChildByInterface( _Switch_INTERFACE ); }

		/**
		 * @brief   Get last known door state (opened, closed, unknown)
		 * @return  current door state (enum)
		 */
		Door::DOOR_STATE getState() const				{ return this->state; }
		/**
		 * @brief   Get door state as a literal string
		 * @return  current door state as literal string
		 */
		inline const char* getStateAsString() const		{ return getStateAsString(this->state); }
		/**
		 * @brief   Get door state as a literal string
		 * @param state [in] door state to be retrieved as literal string
		 * @return  passed door state as literal string
		 */
		static const char* getStateAsString( Door::DOOR_STATE state );
		/**
		 * @brief   Get door state from a literal string
		 * @param state [in] door state as literal string
		 * @return  door state (as enum) associated with the argument 
		 */
		static Door::DOOR_STATE getStateFromString( const char* state );

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	private:
		WRESULT mmDoOpen( const SecurityToken* security );
		WRESULT mmDoClose( const SecurityToken* security );
		WRESULT mmDoLock( const SecurityToken* security );
		WRESULT mmDoUnlock( const SecurityToken* security );

	protected:
		virtual void init();
		void setState( DOOR_STATE state, bool raiseEvent = true, const SecurityToken* source_token = NULL );

	protected:
		Door::DOOR_STATE state;		///< current state of the door


}; // class def



/**
 * @class   wosh::building::Door_StateChanged
 * @brief   Raised when door state is changed (such as open to closed).
 *
 * Internally, data is stored as wosh::List. Supports also wosh::Variant type.
 *
 * Fields are:
 *  - \c door_state [Variant::INTEGER] (always present, first item of the list)
 *  - \c previous_door_state (optional, second item of the list)
 *
 * Some getters are provided for convenience:
 * \code

 * \endcode
 *
 * @see     wosh::Fact
 * @see     wosh::building:Door
 ****************************************************************************
 * @version 0.8.399 $Id: Door.h 2732 2010-07-06 03:18:11Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
class Door_StateChanged : public wosh::Fact
 {
	WOSH_SERIALIZABLE(wosh::building::Door_StateChanged)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Door_StateChanged()
			: wosh::Fact(_Door_EVENT_DoorStateChanged) { }
		/**
		 * @brief  Constructor with initializers.
		 * @param current_door_state [in] current state of the door
		 */
		Door_StateChanged( Door::DOOR_STATE current_door_state );
		/**
		 * @brief  Constructor with initializers.
		 * @param current_door_state [in] current state of the door
		 * @param previous_door_state [in] previous state of the door
		 */
		Door_StateChanged( Door::DOOR_STATE current_door_state, Door::DOOR_STATE previous_door_state );
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Door_StateChanged( const Door_StateChanged& m )
			: wosh::Fact(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Door_StateChanged() { }

	public:
		/**
		 * @brief   Get door state (opened, closed, unknown)
		 * @return  current door state (enum)
		 */
		Door::DOOR_STATE getDoorState() const;

		/**
		 * @brief   Get previous door state (opened, closed, unknown)
		 * @return  previous door state (enum)
		 */
		Door::DOOR_STATE getPreviousDoorState() const;


}; // class def



 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_Door_H__
