/**
 * @class   wosh::building::Sprinkler
 * @brief   BuildingObject representing a generic (real) sprinkler.
 *          This class provides standard Methods and properties to 
 *          control the the mapped wosh::interfaces::devices::Switch (or Dimmer)
 *          wosh::building::Sprinkler is usually a child of wosh::building::Garden
 *
 ****************************************************************************
 * @version $Id: Sprinkler.h 2556 2010-05-26 23:16:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Building_Sprinkler_H__
 #define __WOSH_Building_Sprinkler_H__

 #include <framework/building/BuildingObject.h>

 #include <interfaces/devices/Switch.h>
 #include <interfaces/devices/Dimmer.h>


namespace wosh {
 namespace building {

 #define _Sprinkler_VERSION					0.27

 #define _Sprinkler_KEY_Device				"Device"


class Sprinkler : public BuildingObject
 {
	WOSH_BUILDINGOBJECT(wosh::building::Sprinkler)

	public:
		/**
		 * @brief   type
		 */
		enum SPRINKLER_TYPE {
			TYPE_UNKNOWN		= 0x00,	///< 
			TYPE_IMPACT			= 0x01,	///< 
			TYPE_OSCILLATING	= 0x02,	///< 
			TYPE_DRIP			= 0x04,	///< 
			TYPE_UNDERGROUND	= 0x08	///< 
		};

		/**
		 * @brief   states
		 */
		enum SPRINKLER_STATE {
			STATE_UNKNOWN	= 0x00,	///< 
			STATE_IDLE		= 0x01,	///< 
			STATE_OPENED	= 0x02	///< 
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Sprinkler();
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Sprinkler( const Sprinkler& m ) : BuildingObject(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Sprinkler() { }


	public:
		virtual void busMessageChild( const Message& message );

		virtual WRESULT setChild( const std::string& object_uri, InterfacesProvider* interfaces );

//@}
/** @name Sprinkler Interface - 
 * @{
 ******************************************************************************/
	public:
	//	virtual WRESULT open()					{ return mmDoSetVolumeUp(NULL);}
	//	virtual WRESULT close()					{ return mmDoSetVolumeDown(NULL);}

	public:

		/**
		 * @brief   Get last known audio-box state (opened, closed, unknown)
		 * @return  current audio-box state (enum)
		 */
		Sprinkler::SPRINKLER_STATE getState() const		{ return this->state; }
		/**
		 * @brief   Get audio-box state as a literal string
		 * @return  current audio-box state as literal string
		 */
		inline const char* getStateAsString() const		{ return getStateAsString(this->state); }
		/**
		 * @brief   Get audio-box state as a literal string
		 * @param state [in] audio-box state to be retrieved as literal string
		 * @return  passed audio-box state as literal string
		 */
		static const char* getStateAsString( Sprinkler::SPRINKLER_STATE state );
		/**
		 * @brief   Get audio-box state from a literal string
		 * @param state [in] audio-box state as literal string
		 * @return  audio-box state (as enum) associated with the argument 
		 */
		static Sprinkler::SPRINKLER_STATE getStateFromString( const char* state );

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	private:


	protected:
		virtual void init();
		void setState( Sprinkler::SPRINKLER_STATE state, bool raiseEvent = true, const SecurityToken* source_token = NULL );

	protected:
		Sprinkler::SPRINKLER_STATE state;	///< current state of the audiobox
		Sprinkler::SPRINKLER_TYPE type;

}; // class def




/**
 * @class   wosh::building::Sprinkler_StateChanged
 * @brief   Raised when door state is changed (such as open to closed).
 *
 * Internally, data is stored as wosh::List. Supports also wosh::Variant type.
 *
 * Fields are:
 *  - \c audiobox_state [Variant::INTEGER] (always present, first item of the list)
 *  - \c previous_appliance_state (optional, second item of the list)
 *
 * Some getters are provided for convenience:
 * \code

 * \endcode
 *
 * @see     wosh::Fact
 * @see     wosh::building:Sprinkler
 ****************************************************************************
 * @version 0.8.399 $Id: Sprinkler.h 2556 2010-05-26 23:16:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
class Sprinkler_StateChanged : public wosh::Fact
 {
	WOSH_SERIALIZABLE(wosh::building::Sprinkler_StateChanged)

	public:
		/**
		 * @brief  Empty constructor.
		 */
		Sprinkler_StateChanged()
			: wosh::Fact(_BuildingObject_EVENT_StateChanged) { }
		/**
		 * @brief  Constructor with initializers.
		 * @param current_audiobox_state [in] current state of the appliance
		 */
		Sprinkler_StateChanged( Sprinkler::SPRINKLER_STATE current_audiobox_state );
		/**
		 * @brief  Constructor with initializers.
		 * @param current_audiobox_state [in] current state of the appliance
		 * @param current_volume [in] current level of the appliance
		 */
		Sprinkler_StateChanged( Sprinkler::SPRINKLER_STATE current_audiobox_state, int current_volume );
		/**
		 * @brief  Constructor with initializers.
		 * @param current_audiobox_state [in] current state of the appliance
		 * @param current_volume [in] current level of the appliance
		 * @param previous_appliance_state [in] previous state of the appliance
		 * @param previous_volume [in] previous level of the appliance
		 */
		Sprinkler_StateChanged( Sprinkler::SPRINKLER_STATE current_audiobox_state, int current_volume, Sprinkler::SPRINKLER_STATE previous_appliance_state, int previous_volume );
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		Sprinkler_StateChanged( const Sprinkler_StateChanged& m )
			: wosh::Fact(m) { }
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~Sprinkler_StateChanged() { }

	public:
		/**
		 * @brief   Get appliance state (opened, closed, unknown).
		 * @return  current appliance state (enum)
		 */
		Sprinkler::SPRINKLER_STATE getSprinklerState() const;
		/**
		 * @brief   Get appliance level.
		 * @return  current appliance level (value)
		 */
		int getVolume() const;

		/**
		 * @brief   Get previous appliance state (opened, closed, unknown).
		 * @return  previous appliance state (enum)
		 */
		Sprinkler::SPRINKLER_STATE getPreviousSprinklerState() const;
		/**
		 * @brief   Get appliance level.
		 * @return  current appliance level (value)
		 */
		int getPreviousVolume() const;

		inline bool isOn() const				{ return (getVolume() > 0); }
		inline bool isOff() const				{ return (getVolume() == 0); }

}; // class def





 }; // namespace building
}; // namespace wosh

#endif //__WOSH_Building_Sprinkler_H__
