/**
 * @class   wosh::communication::AudioMessage
 * @brief   wosh::Notification representing an audio message (based on audio-file).
 *          The file containing the audio message is stored in a file
 *          (for example, it may be an mp3).
 *          When message is sent over the network, file is not copied or,
 *          its URI must be valid on all system as the phisical resource.
 *          (see $SHARED_TEMP alias)
 *
 * @see     wosh::interfaces::devices::PlayerAudio
 ****************************************************************************
 * @version $Id: AudioMessage.h 2556 2010-05-26 23:16:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_AudioMessage_H__
 #define __WOSH_Communication_AudioMessage_H__

 #include <woshDefs.h>
 #include <framework/communication/FileMessage.h>


namespace wosh {
 namespace communication {

class AudioMessage : public wosh::communication::FileMessage
 {
	WOSH_SERIALIZABLE(wosh::communication::AudioMessage)
	WOSH_COMPARE_RTTI(wosh::communication::AudioMessage)

	public:

		/// type of audio message
		enum AUDIO_TYPE {
			AUDIO_GENERIC	= 0x00,
			AUDIO_TEXT		= 0x01,
			AUDIO_DMTF		= 0x02,
			AUDIO_MUSIC		= 0x04
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		AudioMessage();
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param filename [in] the (wosh) path of the audio file
		 * @see    setFile()
		 */
		AudioMessage( const std::string& filename );
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		AudioMessage( const AudioMessage& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~AudioMessage() { }

	public:
		virtual bool fromNotification( const Notification* notif );


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		inline void setAudioType( AUDIO_TYPE type )			{ this->audioType = type; }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		inline AUDIO_TYPE getAudioType() const					{ return this->audioType; }

		inline Notification::TYPE getNotificationType() const	{ return Notification::TYPE_AUDIO; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual AudioMessage& operator=(const AudioMessage& m);

		virtual bool operator==(const AudioMessage &other) const;
		virtual bool operator!=(const AudioMessage &other) const { return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		virtual void clear();

	protected:
		AUDIO_TYPE audioType;


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_AudioMessage_H__
