/** @file    SessionShell.cpp
 *  @author  Alessandro Polo
 *  @version $Id: SessionShell.cpp 2621 2010-06-04 12:23:48Z alex $
 *  @brief
 * File containing methods for the wosh::communication::SessionShell class.
 * The header for this class can be found in SessionShell.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/communication/SessionShell.h>

 #include <core/SessionProtocol.h>
 #include <framework/security/SecurityAuthenticationShell.h>
 #include <core/ObjectAllocator.h>
 #include <core/Utilities.h>
 #include <core/WoshKernel.h>
 #include <iostream>
 //#include <termios.h>

 
using namespace std;
namespace wosh {
 namespace communication {

 WOSH_REGISTER(wosh::communication::SessionShell, "wosh::Session", 1.0, _static_SessionShell )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS

SessionShell::SessionShell()
	: ThreadImpl(NULL, "wosh.SessionShell.Thread"), Session() {
	Object::setName("SessionShell", getID());
	Log.setContext( "SessionShell", true );

	this->streamInput = &std::cin;
	this->streamOutput = &std::cout;
 }

SessionShell::~SessionShell() {
	Log(LOG_VERBOSE, ":~SessionShell() Destroying.." );
	if ( isThreadRunning() )
		close();
 }

//////////////////////////////////////////////////////////////////////////////////////////////// CONSTRUCTORS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

WRESULT SessionShell::open( unsigned long timeout ) {
	if ( this->streamInput == NULL || this->streamOutput == NULL ) return WRET_ERR_ILLEGAL_USE;
	if ( getProtocol() == NULL ) return WRET_ERR_ILLEGAL_USE;
	return startThread(timeout);
 }

WRESULT SessionShell::close( unsigned long timeout ) {
	return quitThread(timeout);
 }

void SessionShell::printMessage( const std::string& output_text ) {
	*this->streamOutput << output_text << std::endl;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionShell::initializingThread() {
	if ( this->streamInput == NULL || this->streamOutput == NULL ) return false;
	if ( getProtocol() == NULL ) return false;
	return true;
 }

void SessionShell::runThread()
 {
	while( this->running ) {
		if ( handleLogin(this->running) )
			handleSession(this->running);
		if ( !this->streamInput->good() ) {
			Log(LOG_CRITICAL, ":handleSession() Invalid Input buffer!" );
			return;
		 }
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SessionShell::handleLogin( bool& still_running ) {
	Log(LOG_VERBOSE, ":handleLogin() Logging In.." );
	*this->streamOutput << std::endl;
	char buf[255+1];
	string user, pwd;
	while ( user.size() < 3 ) {
		*this->streamOutput << "UserName: ";
		this->streamInput->getline(buf, 255);
		user.assign(buf);
		Utilities::trimSpaces(user);
		if ( user.empty() || !still_running || !this->streamInput->good() )
			return false;
	 }
	*this->streamOutput << "PassWord: ";
	this->streamInput->getline(buf, 255);
	pwd.assign(buf);
	if ( !still_running || !this->streamInput->good() )
		return false;
	wosh::security::SecurityTokenProviderShell secPwd;//pwd);
	WRESULT ret = login( user, &secPwd );
	if ( WFAILED(ret) )
		return false;
	return true;
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SessionShell::handleSession( bool& still_running ) {
	if ( !getSkipWelcome() ) {
		bool sentok = ( *this->streamOutput << getWelcomeMessage() << std::endl );
		if ( !sentok ) {
			Log(LOG_WARNING, ":runThread() FAILED seding welcome message [continuing]" );
		 }
	 }
	Log(LOG_VERBOSE, ":handleSession() Entering loop.." );
	char buf[255+1];
	std::string deprecated;
	while( still_running && this->streamInput->good() ) {
		*this->streamOutput << getPrompt();
		this->streamInput->getline(buf, 255);
		deprecated.assign(buf);
		parseInput( deprecated );
	 }
	Log(LOG_VERBOSE, ":handleSession() Exited loop.." );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace communication
}; // namespace wosh
