/**
 * @class   wosh::communication::TextMessage
 * @brief   wosh::Notification representing an text message (as SMS, Mail).
 *          
 *
 ****************************************************************************
 * @version $Id: TextMessage.h 2556 2010-05-26 23:16:48Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Communication_TextMessage_H__
 #define __WOSH_Communication_TextMessage_H__

 #include <woshDefs.h>
 #include <core/Notification.h>


namespace wosh {
 namespace communication {

class TextMessage : public Notification
 {
	WOSH_SERIALIZABLE(wosh::communication::TextMessage)
	WOSH_COMPARE_RTTI(wosh::communication::TextMessage)

	public:

		/// type of text message
		enum MESSAGE_TYPE {
			MESSAGE_GENERIC = 0,
			MESSAGE_SMS = 1,
			MESSAGE_IM = 2,
			MESSAGE_MAIL = 4
		};

	public:
		/**
		 * @brief  Empty constructor.
		 */
		TextMessage();
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param plain [in] the plain text of the message
		 * @see    setPlain()
		 */
		TextMessage( const std::string& plain );
		/**
		 * @brief  Constructor with initializer(s), provided for convenience.
		 * @param subject [in] the subject of the message
		 * @param plain [in] the plain text of the message
		 * @see    setSubject()
		 * @see    setPlain()
		 */
		TextMessage( const std::string& subject, const std::string& plain );
		/**
		 * @brief  Copy constructor.
		 * @param m [in] the source object to be copied
		 */
		TextMessage( const TextMessage& m );
		/**
		 * @brief  Empty destructor.
		 */
		virtual ~TextMessage() { }

	public:

		virtual bool fromNotification( const Notification* notif );


//@}
/** @name Setters
 * @{
 ******************************************************************************/
	public:
		void setMessageType( MESSAGE_TYPE type )				{ this->messageType = type; }

		void setSMS()											{ this->messageType = TextMessage::MESSAGE_SMS; }
		void setMail()											{ this->messageType = TextMessage::MESSAGE_MAIL; }
		void setIM()											{ this->messageType = TextMessage::MESSAGE_IM; }

		virtual void setSubject( const std::string& value )		{ this->subject = value; }
		virtual void setPlain( const std::string& value )		{ this->plain = value; }

//@}
/** @name Getters
 * @{
 ******************************************************************************/
	public:
		MESSAGE_TYPE getMessageType() const						{ return this->messageType; }

		bool isSMS() const										{ return (this->messageType == TextMessage::MESSAGE_SMS); }
		bool isMail() const										{ return (this->messageType == TextMessage::MESSAGE_MAIL); }
		bool isIM() const										{ return (this->messageType == TextMessage::MESSAGE_IM); }


		virtual const std::string& getSubject() const			{ return this->subject; }
		virtual std::string& getSubject()						{ return this->subject; }

		virtual const std::string& getPlain() const				{ return this->plain; }
		virtual std::string& getPlain()							{ return this->plain; }

		virtual std::string getSummary( int maxsize = 1023 ) const;

		virtual Notification::TYPE getNotificationType() const	{ return Notification::TYPE_TEXTUAL; }

//@}
/** @name Operators Overloading
 * @{
 ******************************************************************************/
	public:
		virtual TextMessage& operator=(const TextMessage& m);

		virtual bool operator==(const TextMessage &other) const;
		virtual bool operator!=(const TextMessage &other) const	{ return !(*this == other);  }

//@}
/** @name Serialization and XML methods
 * @{
 ******************************************************************************/
	public:
		virtual std::string toString() const;
		virtual bool updateHash( DataHashContext* hash_context ) const;
//@}
	protected:
		virtual void clear();

/** @name Internal vars
 * @{
 ******************************************************************************/
	protected:
		MESSAGE_TYPE messageType;	///< type of the message

		std::string subject;		///< subject of the message
		std::string plain;			///< plain (extended) text of the message

//@}


}; // class def

 }; // namespace communication
}; // namespace wosh

#endif //__WOSH_Communication_TextMessage_H__
