/** @file    GuiDialogs.cpp
 *  @author  Alessandro Polo
 *  @version $Id: GuiDialogs.cpp 2488 2010-05-21 01:48:27Z alex $
 *  @brief
 * File containing methods for the dialog classes.
 * The header for this class can be found in GuiDialogs.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include <framework/gui/dialogs/GuiDialogs.h>
 #include <framework/gui/GuiUtilities.h>

 #include <QtGui>

 #include "ui_DialogGuiMapping.h"
 


using namespace std;
namespace wosh {
 namespace gui {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GuiMappingTableModel::rowCount( const QModelIndex &parent ) const {
	(void)parent;
	return GuiMappings::getMappings().count();
 }


QVariant GuiMappingTableModel::data(const QModelIndex &index, int role) const {
	if (  !index.isValid() ) return QVariant();
	if ( role == Qt::DisplayRole || role == Qt::EditRole ) {
		MutexLockerRead mL(GuiMappings::getMappings().getMutex());
		MapT<std::string, ObjectTypeInfo>::ConstIterator it;
		it = GuiMappings::getMappings().itemAt(index.row());
		if ( it == GuiMappings::getMappings().end() )
			return QVariant();
		const std::string& guiType = it->first;
		const ObjectTypeInfo& info = it->second;
		switch(index.column()) {
			case 0: return QVariant( guiType.c_str() );
			case 1: return QVariant( info.name.c_str() );
			case 2: return QVariant( info.version );
			default: return QVariant();
		 }
	 }
	else if ( role == Qt::BackgroundColorRole ) { }
	else if ( role == Qt::ToolTipRole ) { }
	return QVariant();
 }

QVariant GuiMappingTableModel::headerData(int section, Qt::Orientation orientation, int role) const {
	if (role != Qt::DisplayRole) return QVariant();
	 if (orientation == Qt::Horizontal ) {
		switch(section) {
			case 0: return QVariant("ClassType");
			case 1: return QVariant("GUI ClassType");
			case 2: return QVariant("Version");
			default: return QVariant();
		 }
	 }
	return QVariant();
 }

DialogGuiMapping::DialogGuiMapping( QWidget *parent ) : QDialog(parent) {
	this->m_ui = new Ui::DialogGuiMapping();
	this->m_ui->setupUi(this);

	this->tableModel = new GuiMappingTableModel(this);
	this->m_ui->tableView->setModel(this->tableModel);

	QObject::connect(this->m_ui->btnClose, SIGNAL(clicked()), this, SLOT(accept()));
	QObject::connect(this->m_ui->btnRefresh, SIGNAL(clicked()), this, SLOT(refreshData()));

	refreshData();
 }

DialogGuiMapping::~DialogGuiMapping() {
	delete this->m_ui;
 }

void DialogGuiMapping::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

void DialogGuiMapping::refreshData() {
	this->tableModel->reset();
	this->m_ui->tableView->resizeColumnsToContents();
	this->m_ui->tableView->horizontalHeader()->setResizeMode(0,QHeaderView::Stretch);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////












DialogSerializableEditor::DialogSerializableEditor( QWidget *parent )
	: QDialog(parent) {

	this->obj_gui = NULL;

	this->vLayout = new QVBoxLayout(this);
	this->vLayout->setContentsMargins(0,0,0,0);
	this->vLayout->setObjectName(QString::fromUtf8("verticalLayout"));

	this->buttonBox = new QDialogButtonBox(this);
	this->buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
	this->buttonBox->setOrientation(Qt::Horizontal);
	this->buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

	this->vLayout->addWidget(this->buttonBox);

	this->resize(300, 100);

	QObject::connect(this->buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
	QObject::connect(this->buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
 }

void DialogSerializableEditor::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
//	if ( e->type() == QEvent::LanguageChange )
//		this->retranslateUi(this);
 }

WRESULT DialogSerializableEditor::loadObject( ISerializable* object ) {
	if ( object == NULL ) return WRET_ERR_PARAM;

	if ( this->obj_gui != NULL ) {
		if ( !this->obj_gui->isSerializableSupported(object) ) {
			this->vLayout->removeWidget( this->obj_gui->asQWidget() );
			delete this->obj_gui; this->obj_gui = NULL;
		 }
		else {
			return this->obj_gui->loadSerializable(object);
		 }
	 }

	this->obj_gui = GuiUtilities::createGuiFor<IWidgetSerializable>(object->getClassName());
	if ( this->obj_gui == NULL ) {
		QMessageBox::warning(this, tr("Create GUI.."), tr("FAILED. No Widget found.") );
		return WRET_ERR_INTERNAL;
	 }
	WRESULT ret = this->obj_gui->loadSerializable(object);
	this->vLayout->insertWidget( 0, this->obj_gui->asQWidget() );
	return ret;
 }

WRESULT DialogSerializableEditor::saveObject( ISerializable* object ) {
	if ( object == NULL ) return WRET_ERR_PARAM;
	if ( this->obj_gui == NULL ) return WRET_ERR_ILLEGAL_USE;
	return this->obj_gui->saveSerializable(object);
 }		

//void DialogSerializableEditor::accept() {
//	QDialog::accept();
// }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
