/** @file    FormSecurityManager.cpp
 *  @author  Alessandro Polo
 *  @version $Id: FormSecurityManager.cpp 2748 2010-07-11 22:07:06Z alex $
 *  @brief
 * File containing methods for the wosh::gui::FormSecurityManager class.
 * The header for this class can be found in FormSecurityManager.h, check that file
 * for class description.
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

 #include "FormSecurityManager.h"
 #include "ui_FormSecurityManager.h"
 #include <core/SecurityManager.h>

 #include <QtGui/QCompleter>
 #include <QtGui/QMessageBox>


using namespace std;
namespace wosh {
 namespace gui {

 WOSH_REGISTER(wosh::gui::FormSecurityManager, "wosh::gui::IWidgetGeneric", 1.0, _static_FormSecurityManager )
 WOSH_GUI_MAP(wosh::gui::FormSecurityManager, "wosh::SecurityManager", 0.0, _static_Map_FormSecurityManager )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

FormSecurityManager::FormSecurityManager(QWidget *parent) : IWidgetGeneric(parent)
 {
	this->m_ui = new Ui::FormSecurityManager();
	this->m_ui->setupUi(this);

	IWidgetGeneric::setTargetURI(_SecurityManager_URI);
	buildMenus();

	// tab EventsLog [0]
	this->m_ui->tableViewEventsLog->setMethodList(_SecurityManager_METHOD_eventslog);
	this->m_ui->tableViewEventsLog->setResizeStretch(1);
	this->m_ui->tableViewEventsLog->getTableView()->setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this->m_ui->tableViewEventsLog, SIGNAL(postMessage(Message*)),this, SLOT(postMessage(Message*)) );
	connect(this->m_ui->tableViewEventsLog, SIGNAL(tableContextMenuRequested(const QPoint&, const QModelIndex&)),this, SLOT(tableViewEventsLog_ContextMenu(const QPoint&, const QModelIndex&)) );
	this->mapMethodButton(_SecurityManager_METHOD_clear_log, this->m_ui->btnClearLog);
	connect(this->m_ui->chkMonitorSecurityBus, SIGNAL(clicked()), this, SLOT(chkMonitorSecurityBus_clicked()));
	this->propMapperCfg = new PropertiesMapper(this, true);
	connect(this->propMapperCfg, SIGNAL(send_request(const QString&, ISerializable*)), this, SLOT(executeRemoteMethod(const QString&, ISerializable*)));
	this->propMapperCfg->set(_SecurityManager_KEY_EventLogMaxSize, this->m_ui->spinEventLogSize);
	this->propMapperCfg->set(_SecurityManager_KEY_EventLogLocallyOnly, this->m_ui->chkRecordLocalEvents);

	// tab Fake Event [2]
	QCompleter* completerURI = new QCompleter(this);
	completerURI->setCaseSensitivity(Qt::CaseInsensitive);
	completerURI->setModel( this->m_ui->tableViewEventsLog->getTableModel() );
	completerURI->setCompletionColumn(1);
	this->m_ui->txtEmulateSource->setCompleter(completerURI);
	QCompleter* completerEvent = new QCompleter(this);
	completerEvent->setCaseSensitivity(Qt::CaseInsensitive);
	completerEvent->setModel( this->m_ui->tableViewEventsLog->getTableModel() );
	completerEvent->setCompletionColumn(2);
	this->m_ui->txtEmulateEventName->setCompleter(completerEvent);
	QCompleter* completerData = new QCompleter(this);
	completerData->setCaseSensitivity(Qt::CaseInsensitive);
	completerData->setModel( this->m_ui->tableViewEventsLog->getTableModel() );
	completerData->setCompletionColumn(3);
	this->m_ui->txtEmulateData->setCompleter(completerData);
	connect(this->m_ui->btnEmulateEvent, SIGNAL(clicked()), this, SLOT(btnEmulateEvent_clicked()));
}

FormSecurityManager::~FormSecurityManager() {
	delete this->m_ui;
 }

void FormSecurityManager::changeEvent(QEvent *e){
	QWidget::changeEvent(e);
	if ( e->type() == QEvent::LanguageChange )
		this->m_ui->retranslateUi(this);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::refreshAll() {
	this->setWindowTitle( QString("Security Manager [%1]").arg(this->getTargetURI().getKernelName().c_str()) );
	this->m_ui->tableViewEventsLog->refreshData();
	refreshConfiguration();
 }

void FormSecurityManager::refreshConfiguration() {
	List* dlProps = this->propMapperCfg->createKeysList();
	executeRemoteMethod( _METHOD_GetProperties, dlProps );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::btnEmulateEvent_clicked() {
	List* dl = new List(3);
	dl->push_back<Variant>( this->m_ui->txtEmulateSource->text().toLatin1().data() );
	dl->push_back<Variant>( this->m_ui->txtEmulateEventName->text().toLatin1().data() );
	dl->push_back<Variant>( this->m_ui->txtEmulateData->text().toLatin1().data() );
	executeRemoteMethod( _SecurityManager_METHOD_fake_event, dl );
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::chkMonitorSecurityBus_clicked() {
	if ( this->m_ui->chkMonitorSecurityBus->isChecked() ) {
		BusSecurity.setMessageHandler(this);
		WRESULT bus_d_connected = BusSecurity.connect( _Bus_Devices );
		if ( WFAILED(bus_d_connected) )
			QMessageBox::warning(this, this->windowTitle(), tr("FAILED#%i Connecting Security Bus!").arg(bus_d_connected) );
	 }
	else {
		WRESULT bus_d_disconnected = BusSecurity.disconnect();
		if ( WFAILED(bus_d_disconnected) )
		QMessageBox::warning(this, this->windowTitle(), tr("FAILED#%i Disconnecting Security Bus!").arg(bus_d_disconnected) );
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::buildMenus() {
	this->menuEventLogTitle = new QAction(this);
	this->menuEventLogTitle->setEnabled(false);
	this->menuEventLogRePlay = new QAction(this);
	this->menuEventLogRePlay->setText(tr("&Re-Play Event"));
	this->menuEventLogFillSheet = new QAction(this);
	this->menuEventLogFillSheet->setText(tr("&Fill Emulate-Event sheet.."));
	this->menuEventLog = new QMenu(this);
	this->menuEventLog->addAction(this->menuEventLogTitle);
	this->menuEventLog->addSeparator();
	this->menuEventLog->addAction(this->menuEventLogRePlay);
	this->menuEventLog->addAction(this->menuEventLogFillSheet);
 }

void FormSecurityManager::tableViewEventsLog_ContextMenu( const QPoint& pos, const QModelIndex& index ) {
	QPoint globalPos = this->mapToGlobal(pos);
	if ( !index.isValid() || index.row() == 0 ) return;
	const URI* deviceURI = this->m_ui->tableViewEventsLog->getTableView()->getItem<URI>(index.row(), 1);
	Variant deviceName("-");
	if ( deviceURI != NULL )
		deviceName = deviceURI->getName();
	this->menuEventLogTitle->setText( deviceName.toString().c_str() );
	//this->menuEventLogTitle->setIcon( GuiUtilities::createIconFor(_Object_TYPE) );

	QAction* selected = this->menuEventLog->exec( globalPos );
	if ( selected == this->menuEventLogRePlay ) {
		Variant eventName = this->m_ui->tableViewEventsLog->getCurrentItemAsVariantAtCol(2);
		Variant eventData = this->m_ui->tableViewEventsLog->getCurrentItemAsVariantAtCol(3);
		List* dl = new List(3);
		dl->push_back( deviceName.clone() );
		dl->push_back( eventName.clone() );
		dl->push_back( eventData.clone() );
		executeRemoteMethod( _SecurityManager_METHOD_fake_event, dl );
	 }
	else if ( selected == this->menuEventLogFillSheet ) {
		Variant eventName = this->m_ui->tableViewEventsLog->getCurrentItemAsVariantAtCol(2);
		Variant eventData = this->m_ui->tableViewEventsLog->getCurrentItemAsVariantAtCol(3);

		this->m_ui->txtEmulateSource->setText( deviceURI->toString().c_str() );
		this->m_ui->txtEmulateEventName->setText( eventName.toString().c_str() );
		this->m_ui->txtEmulateData->setText( eventData.toString().c_str() );
		this->m_ui->tabWidget->setCurrentIndex(2);
	 }
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FormSecurityManager::busMessage( const Message& message, const Bus* ) {
	if ( message.isEmpty() || !message.getContent()->isResponse() ) return;
	if ( !isMessageForMeOrFromTarget(message) ) return;
	const wosh::Response* response = message.getContent()->asResponse();

	this->m_ui->tableViewEventsLog->evalResponse(response);

	if ( response->getMethod() == _SecurityManager_METHOD_fake_event ) {
		if ( WFAILED(response->getReturnValue()) )
			QMessageBox::warning(this, this->windowTitle(), tr("FAILED#%i sending Security Event!").arg(response->getReturnValue()) );
		else {
			this->m_ui->tableViewEventsLog->refreshData();
			QMessageBox::information(this, this->windowTitle(), tr("Security Event has been sent!") );
		 }
		return;
	 }

	this->propMapperCfg->evalResponse(response);

	if ( response->getMethod() == _SecurityManager_METHOD_clear_log && WSUCCEEDED(response->getReturnValue()) )
		this->m_ui->tableViewEventsLog->refreshData();

	promptMethodResponse(message);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

 }; // namespace gui
}; // namespace wosh
