/**
 * @file    framework/network.h
 * @brief   Include all components of the network framework
 ****************************************************************************
 * @version 0.8.161 $Id: network.h 2258 2010-04-02 13:02:21Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/*! @namespace wosh::network
 * @brief   Specification and some implementations of common sockets (TCP, UDP) and networking-related classes. 
 *          WOSH Network Framework is required in order to build a (distributed) WOSH network,
 *          protocols are (installed and) configured as Bundles (within NetworkManager settings).
 *          Thanks to wosh::services::DiscoveryBundle service, a WOSH application may auto-connect
 *          to other host without user interaction.
 * @see     Refer to wosh::NetworkChannel, wosh::NetworkProtocol interfaces and wosh::NetworkManager core-module.
 * @see     \ref page_framework_network
 *
 ****************************************************************************/
/*! \page page_framework
 * \htmlonly <hr/> \endhtmlonly
 * \section page_framework_network WOSH Network Framework
 *
 * wosh::network namespace offers a set of multi-platform classes for networking.
 *
 * The base blocks of the set are socket and server objects such as:
 *  - wosh::network::SocketUdp
 *
 *  - wosh::network::ServerTcp
 *  - wosh::network::SocketTcp
 *
 * You may use this classes within your bundles.
 * Current implementation is based on QT4 Networking classes.
 *
 * wosh::services::DiscoveryBundle transmits and receives UDP datagrams using
 * the SocketUdp class.
 * \code
void sentDatagram() {
	char data[128];
	memset(&data, '\0', 128);
	string addressport = "192.168.0.255:5634";
	int byteSent = this->udpSocket.sendDatagram( addressport, &data, 128 );
 }

void DiscoveryImpl::udpIncomingDatagram( const char * data, int maxSize, const string& source, SocketUdp* sourceSocket ) {
	// evaluate incoming -data- from -source- (address:port)
 }

 * \endcode
 *
 *
 * [..]
 *
 * wosh::application::NetworkProtocolAllocatorStatic is an helper class
 * which maps type-name to its implementation and allocates it.
 *
 *
 * Refer to wosh::NetworkChannel, wosh::NetworkProtocol and wosh::NetworkManager.
 *
 *
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_FrameWork_Network_H__
 #define __WOSH_FrameWork_Network_H__

 #include <woshDefs.h>

namespace wosh {
 namespace network { };
};

 #include <framework/network/ChannelTcpServer.h>
 #include <framework/network/ChannelTcpSocket.h>
 #include <framework/network/ChannelUdpSocket.h>

 #include <framework/network/NetworkProtocolDBus.h>
 #include <framework/network/NetworkProtocolTcp.h>
 #include <framework/network/NetworkProtocolUdp.h>
 #include <framework/network/NetworkUtilities.h>

 #include <framework/network/ServerTcp.h>
 #include <framework/network/SocketTcp.h>
 #include <framework/network/SocketUdp.h>


#endif //__WOSH_FrameWork_Network_H__
