/**
 * @class   wosh::network::ClientHttp
 * @brief   The class provides a generic HTTP client.
 *          Perform HTTP Request and retrieve server Response.
 *          
 *
 ****************************************************************************
 * @version $Id: ClientHttp.h 2802 2010-07-25 15:04:52Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Network_ClientHttp_H__
 #define __WOSH_Network_ClientHttp_H__

 #include <woshDefs.h>
 #include <core/LogContext.h>
 #include <core/FilePath.h>
 #include <core/ResourcePath.h>


 #include <QObject>
class QHttp;
class QHttpResponseHeader;
class QFile;

namespace wosh {
 namespace network {

class ClientHttp;

class IClientHttpListener  {

	public:
		virtual void httpRequestInitiated( wosh::network::ClientHttp* source ) = 0;

		virtual void httpRequestCompleted( wosh::network::ClientHttp* source ) = 0;
		virtual void httpRequestError( wosh::network::ClientHttp* source ) = 0;

		virtual void httpDataReadProgress( long bytesRead, long totalBytes, wosh::network::ClientHttp* source ) = 0;

	public:
		virtual ~IClientHttpListener() { }

}; // class def


class ClientHttp : private QObject {

	Q_OBJECT

	public:

		ClientHttp();

		virtual ~ClientHttp();

	public:

		WRESULT downloadFile();
		WRESULT downloadFile( const wosh::ResourcePath& value, const wosh::FilePath& localPath = FilePath() );

		WRESULT cancelDownload();


		void setListener( wosh::network::IClientHttpListener* ptr )	{ this->listener = ptr; }

		bool isDownloading() const;
		bool isRequestAborted() const	{ return this->httpRequestAborted; }



	public:

		WRESULT setHttpUrl( const wosh::ResourcePath& value, const std::string& username = "", const std::string& password = "" );
		WRESULT setTargetPath( const wosh::FilePath& value );


		const wosh::ResourcePath& getHttpUrl() const		{ return this->httpUrl; }

		const wosh::FilePath& getTargetPath() const		{ return this->targetPath; }


	protected slots:
		void httpRequestStarted( int requestId );
		void httpRequestFinished( int requestId, bool error );
		void done( bool error );

		void readResponseHeader( const QHttpResponseHeader &responseHeader );
		void updateDataReadProgress( int bytesRead, int totalBytes );


	protected:
		wosh::LogContext Log;

		wosh::network::IClientHttpListener* listener;

		wosh::ResourcePath httpUrl;
		wosh::FilePath targetPath;
		std::string httpUsername;
		std::string httpPassword;

		QHttp* httpSocket;
		QFile* httpFile;
		int httpGetId;
		bool httpRequestAborted;

}; // class def

 }; // namespace network
}; // namespace wosh

#endif //__WOSH_Network_ClientHttp_H__
