/**
 * @file    interfaces/services.h
 * @brief   Include all interfaces of wosh::interfaces::Service type
 ****************************************************************************
 * @namespace wosh::interfaces::services
 * @brief     Contains standard interfaces of WOSH services, a convenient classification of wosh::Bundle.
 *
 * wosh::interfaces::Service is a sub-class of a generic wosh::interfaces::IBundle,
 * it represent a generic service.
 *
 * Implementations are usually multi-platform and provide specific functionalities
 * as standalone or interactive software.
 *
 * Often services are 'singleton' on a single host or the whole WOSH network.
 *
 * See \ref Services page for official services.
 * 
 * @see       \ref page_core_bundles
 * @see       wosh::interfaces::Service
 * @see       wosh::interfaces::devices
 * @ingroup   Interfaces
 ****************************************************************************
 * \defgroup Services WOSH Services
 *
 * This page groups all official \c Bundles classified as \b Service, they implement
 * a sub-class of wosh::interfaces::Service.
 *
 * Bundles of this list (classified as \c Service) are usually a generic
 * implementation, multiplatform, singleton (on single host).
 * However such services are considered just as a generic wosh::Bundle.
 *
 * List of official bundles organized by category:
 *
 *  - System Services
 *    - \ref wosh::services::DiscoveryUdpBundle "DiscoveryUdp" - enables network-discovery of WOSH hosts using UDP protocol.
 *    - \ref wosh::services::ManBundle "Man" - Provides manual and built-in documentation of WOSH components.
 *    - \ref wosh::services::SystemServicesBundle "SystemServices" - standardized wrapper to local OS services
 *    - \ref wosh::services::TelnetServerBundle "TelnetServer" - Supports remote logins and shells using \c telnet protocol.
 *
 *  - Mutimedia / Entertainment
 *    - \ref wosh::services::MediaDirectorBundle "MediaDirector" - Centralized service designed to manage entertainment and multimedia of whole house.
 *
 *  - Communication / Messaging
 *    - \ref wosh::services::CommunicationManagerBundle "CommunicationManager" - support (smart and persistent) communication between WOSH users.
 *    - \ref wosh::services::DesktopNotifierBundle "DesktopNotifier" - \ref wosh::interfaces::services::CommunicatorService "Communicator Service" for desktops (trayicon software).
 *    - \ref wosh::services::FestivalBundle "Festival" - \ref wosh::interfaces::services::InterpreterService "Interpreter Service" adapter to \c Festival POSIX software. Enables Text2Speech.
 *
 *  - Home and appliances management
 *    - \ref wosh::services::BuildingManagerBundle "BuildingManager" - Manages home (virtual) representation and access holding object of the \ref wosh::building "WOSH Building framework".
 *    - \ref wosh::services::ButlerBundle "Butler" - Emulate a \c butler human, taking care of the house for you.
 *    - \ref wosh::services::HeyuBundle "Heyu" - adapter to \c heyu software (POSIX only) for controlling X10 devices.
 *    - \ref wosh::services::TaskManagerBundle "TaskManager" - Manages tasks and automations within WOSH, see \ref wosh::automations "WOSH Automation Framework".
 *
 * @see     \ref page_core_bundles, wosh::BundleManager
 * @see     wosh::interfaces::services
 ****************************************************************************
 * @version 0.8.499 $Id: services.h 2288 2010-04-09 08:33:26Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Interfaces_Services_H__
 #define __WOSH_Interfaces_Services_H__

 #include <woshDefs.h>

 namespace wosh {
  namespace interfaces {
   namespace services { };
  };
 };

 #include <interfaces/Service.h>

 #include <interfaces/services/Alarm.h>
 #include <interfaces/services/BigBrother.h>
 #include <interfaces/services/BuildingManager.h>
 #include <interfaces/services/Butler.h>
 #include <interfaces/services/CommunicationManager.h>
 #include <interfaces/services/CommunicatorService.h>
 #include <interfaces/services/DesktopNotifier.h>
 #include <interfaces/services/Discovery.h>
 #include <interfaces/services/DiscoveryUdp.h>
 #include <interfaces/services/Festival.h>
 #include <interfaces/services/Heyu.h>
 #include <interfaces/services/HumanInterpreter.h>
 #include <interfaces/services/InterpreterService.h>
 #include <interfaces/services/Man.h>
 #include <interfaces/services/MediaDirector.h>
 #include <interfaces/services/Organizer.h>
 #include <interfaces/services/SystemServices.h>
 #include <interfaces/services/TaskManager.h>
 #include <interfaces/services/TelnetServer.h>
 #include <interfaces/services/Weather.h>


#endif //__WOSH_Interfaces_Services_H__
