/**
 * @class   wosh::interfaces::services::CommunicatorService
 * @brief   Standard interface of Communicator (service).
 *
 * @ingroup Interfaces
 ****************************************************************************
 * @version 0.8.384 $Id: CommunicatorService.h 2741 2010-07-07 18:51:20Z alex $
 * @author  Alessandro Polo
 ****************************************************************************/
/* Copyright (c) 2007-2010, WOSH - Wide Open Smart Home 
 * by Alessandro Polo - OpenSmartHome.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSmartHome.com WOSH nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Alessandro Polo ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Alessandro Polo BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef __WOSH_Interfaces_Services_CommunicatorService_H__
 #define __WOSH_Interfaces_Services_CommunicatorService_H__

 #include <interfaces/Service.h>


namespace wosh {
 class Notification;
 class Message;

 namespace interfaces {
  namespace services {

 #define _CommunicatorService_TYPE						"wosh::interfaces::services::CommunicatorService"

 #define _CommunicatorService_KEY_ChannelType			"ChannelType"
 #define _CommunicatorService_KEY_ChannelLink			"ChannelLink"
 #define _CommunicatorService_KEY_Cost					"MessageCost"
 #define _CommunicatorService_KEY_Priority				"Priority"
 #define _CommunicatorService_KEY_Reliability			"Reliability"

 #define _CommunicatorService_KEY_Protocol				"Protocol"
 #define _CommunicatorService_KEY_Locations				"Locations"

 #define _CommunicatorService_KEY_RaisePresenceChanged	"RaiseEvent_PresenceChanged"
 #define _CommunicatorService_KEY_RaiseUserInteraction	"RaiseEvent_UserInteraction"

 #define _CommunicatorService_RESPONSE_notif_sent		"notif_sent"
 #define _CommunicatorService_RESPONSE_notif_read		"notif_read"

 #define _CommunicatorService_METHOD_declare			"declare"
 #define _CommunicatorService_METHOD_sendmessage		"sendmessage"

 #define _CommunicatorService_EVENT_PresenceChanged		"PresenceChanged"
 #define _CommunicatorService_EVENT_UserInteraction		"UserInteraction"
 #define _CommunicatorService_EVENT_UserLogging			"UserLogging"


class CommunicatorService : public virtual wosh::interfaces::Service
 {
	WOSH_BUNDLE_INTERFACE("wosh::interfaces::services::CommunicatorService")


/** @name CommunicatorService Interface
 * @{
 ******************************************************************************/
	public:

		// float get_message_cost();
	
		/**
		 * @brief   Check if user is reachable (means online in most cases)
		 * @param wosh_username [in] wosh-username to lookup
		 * @return  true if user is online; false else
		 */
		virtual bool isUserReachable( const std::string& wosh_username ) = 0;

		/**
		 * @brief   Send a text message to given (wosh) username
		 * @param wosh_username [in] recipent of the message (as WOSH username)
		 * @param message [in] message to be sent
		 * @return WRET_ERR_PARAM if account wasn't found, WRET_OK when message was sent, WRET_ERR_INTERNAL else.
		 */
		virtual WRESULT sendMessageToUser( const std::string& wosh_username, const std::string& message ) = 0;

		/**
		 * @brief   Send a text message to given account name [example: asd123@gmail.com]
		 * @param custom_username [in] recipent of the message
		 * @param message [in] message to be sent
		 * @return WRET_ERR_PARAM if account wasn't found, WRET_OK when message was sent, WRET_ERR_INTERNAL else.
		 */
		virtual WRESULT sendMessageTo( const std::string& custom_username, const std::string& message ) = 0;

		/**
		 * @brief   Send a Notification translated into a text message
		 * @param notification [in] the Notification to communicate
		 * @param notification_message [in] message containing the Notification to communicate
		 * @return WRET_ERR_PARAM if account wasn't found, WRET_OK when message was sent, WRET_ERR_INTERNAL else.
		 */
		virtual WRESULT notify( const Notification* notification, const Message* notification_message = NULL ) = 0;

//@}
/*

		/// describes the supported service {as messaging, phone, video}. See CHANNEL_TYPE
		virtual int getChannelType() = 0;

		///< describes the communication channel {synch, asynch}
		virtual CHANNEL_LINK getChannelLink() = 0;

		///< cost of the service,   / message
		virtual float getMessageCost() = 0;

		///< priority of the service
		virtual int getPriority() = 0;

		///< reliability of the service
		virtual int getReliability() = 0;

*/

	public:
		virtual ~CommunicatorService() { }

}; // class def

  }; // namespace services
 }; // namespace interfaces
}; // namespace wosh

#endif //__WOSH_Interfaces_Services_CommunicatorService_H__
